@@ -34,6 +34,7 @@
 
 #include "client/client_priv.h"
 #include "compression.h"
+#include "m_string.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
 #include "my_default.h"
@@ -42,8 +43,11 @@
 #include "my_macros.h"
 #include "my_systime.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/int2str.h"
 #include "mysql_version.h"
+#include "nulls.h"
 #include "print_version.h"
+#include "strxmov.h"
 #include "thr_cond.h"
 #include "thr_mutex.h"
 #include "typelib.h"
