@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,6 +23,7 @@
 #ifndef CERTIFIER_INCLUDE
 #define CERTIFIER_INCLUDE
 
+#include <assert.h>
 #include <mysql/group_replication_priv.h>
 #include <list>
 #include <map>
@@ -30,7 +31,6 @@
 #include <unordered_map>
 #include <vector>
 
-#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "plugin/group_replication/include/certifier_stats_interface.h"
 #include "plugin/group_replication/include/gcs_plugin_messages.h"
@@ -57,7 +57,7 @@ class Gtid_set_ref : public Gtid_set {
         reference_counter(0),
         parallel_applier_sequence_number(parallel_applier_sequence_number) {}
 
-  virtual ~Gtid_set_ref() {}
+  virtual ~Gtid_set_ref() = default;
 
   /**
     Increment the number of references by one.
@@ -72,7 +72,7 @@ class Gtid_set_ref : public Gtid_set {
     @return the number of references
   */
   size_t unlink() {
-    DBUG_ASSERT(reference_counter > 0);
+    assert(reference_counter > 0);
     return --reference_counter;
   }
 
@@ -107,8 +107,6 @@ class Gtid_set_ref : public Gtid_set {
   negatively certified. Otherwise, this transaction is marked
   certified and goes into applier.
 */
-typedef std::unordered_map<std::string, Gtid_set_ref *> Certification_info;
-
 class Certifier_broadcast_thread {
  public:
   /**
@@ -173,7 +171,7 @@ class Certifier_broadcast_thread {
 
 class Certifier_interface : public Certifier_stats {
  public:
-  ~Certifier_interface() override {}
+  ~Certifier_interface() override = default;
   virtual void handle_view_change() = 0;
   virtual int handle_certifier_data(
       const uchar *data, ulong len,
@@ -183,15 +181,23 @@ class Certifier_interface : public Certifier_stats {
       std::map<std::string, std::string> *cert_info) = 0;
   virtual int set_certification_info(
       std::map<std::string, std::string> *cert_info) = 0;
+  virtual int stable_set_handle() = 0;
   virtual bool set_group_stable_transactions_set(
       Gtid_set *executed_gtid_set) = 0;
   virtual void enable_conflict_detection() = 0;
   virtual void disable_conflict_detection() = 0;
   virtual bool is_conflict_detection_enable() = 0;
+  virtual ulonglong get_certification_info_size() override = 0;
 };
 
 class Certifier : public Certifier_interface {
  public:
+  typedef std::unordered_map<
+      std::string, Gtid_set_ref *, std::hash<std::string>,
+      std::equal_to<std::string>,
+      Malloc_allocator<std::pair<const std::string, Gtid_set_ref *>>>
+      Certification_info;
+
   Certifier();
   ~Certifier() override;
 
@@ -332,12 +338,12 @@ class Certifier : public Certifier_interface {
   void get_last_conflict_free_transaction(std::string *value) override;
 
   /**
-    Generate group GNO for a view change log event.
+    Generate group GTID for a view change log event.
 
-    @retval  >0         view change GNO
-    @retval  otherwise  Error on GNO generation
+    @retval  >0         view change GTID
+    @retval  otherwise  Error on GTID generation
   */
-  rpl_gno generate_view_change_group_gno();
+  Gtid generate_view_change_group_gtid();
 
   /**
     Public method to add the given gno value to the group_gtid_executed set
@@ -347,12 +353,11 @@ class Certifier : public Certifier_interface {
                     group_gtid executed GTID set. The sidno used for this
     transaction will be the group_sidno. The gno here belongs specifically to
     the group UUID.
-    @param[in] local If the gtid value is local or comes from a remote server
 
     @retval  1  error during addition.
     @retval  0  success.
   */
-  int add_group_gtid_to_group_gtid_executed(rpl_gno gno, bool local);
+  int add_group_gtid_to_group_gtid_executed(rpl_gno gno);
 
   /**
     Public method to add the given GTID value in the group_gtid_executed set
@@ -360,13 +365,21 @@ class Certifier : public Certifier_interface {
 
     @param[in] gle  The gtid value that needs to the added in the
                     group_gtid_executed GTID set.
-    @param[in] local If the gtid value is local or comes from a remote server
 
     @retval  1  error during addition.
     @retval  0  success.
   */
-  int add_specified_gtid_to_group_gtid_executed(Gtid_log_event *gle,
-                                                bool local);
+  int add_specified_gtid_to_group_gtid_executed(Gtid_log_event *gle);
+
+  /**
+    Computes intersection between all sets received, so that we
+    have the already applied transactions on all servers.
+
+    @return the operation status
+      @retval 0      OK
+      @retval !=0    Error
+  */
+  int stable_set_handle() override;
 
   /**
     This member function shall add transactions to the stable set
@@ -382,17 +395,6 @@ class Certifier : public Certifier_interface {
    */
   bool set_group_stable_transactions_set(Gtid_set *executed_gtid_set) override;
 
-  /**
-    Method to get a string that represents the last local certified GTID
-
-    @param[out] local_gtid_certified_string  The last local GTID transaction
-    string
-
-    @retval 0    if there is no GTID / the string is empty
-    @retval !=0  the size of the string
-  */
-  size_t get_local_certified_gtid(std::string &local_gtid_certified_string);
-
   /**
     Enables conflict detection.
   */
@@ -429,6 +431,15 @@ class Certifier : public Certifier_interface {
   */
   rpl_sidno group_gtid_sid_map_group_sidno;
 
+  /**
+    The sidno used for view log events as seen by the group sid map
+    */
+  rpl_sidno views_sidno_group_representation;
+  /**
+    The sidno used for view log events as seen by the server sid map
+    */
+  rpl_sidno views_sidno_server_representation;
+
   /**
     Method to initialize the group_gtid_executed gtid set with the server gtid
     executed set and applier retrieved gtid set values.
@@ -488,10 +499,26 @@ class Certifier : public Certifier_interface {
 
     @param[in] gno  rpl_gno part of the executing gtid of the ongoing
                     transaction.
-    @param[in] local_transaction if the GTID belongs to a local transaction
   */
-  void add_to_group_gtid_executed_internal(rpl_sidno sidno, rpl_gno gno,
-                                           bool local_transaction);
+  void add_to_group_gtid_executed_internal(rpl_sidno sidno, rpl_gno gno);
+
+  /**
+    This method is used to get the next valid GNO for the given sidno,
+    for the transaction that is being executed. It checks the already
+    used up GNOs and based on that chooses the next possible value.
+    This method will consult group_available_gtid_intervals to
+    assign GTIDs in blocks according to gtid_assignment_block_size
+    when `sidno` is the group sidno.
+
+    @param member_uuid        The UUID of the member from which this
+                              transaction originates. It will be NULL
+                              on View_change_log_event.
+    @param sidno              The sidno that will be used on GTID
+
+    @retval >0                The GNO to be used.
+    @retval -1                Error: GNOs exhausted for group UUID.
+  */
+  rpl_gno get_next_available_gtid(const char *member_uuid, rpl_sidno sidno);
 
   /**
     This method is used to get the next valid GNO for the
@@ -516,6 +543,7 @@ class Certifier : public Certifier_interface {
     This method will consult group_gtid_executed to avoid generate
     the same value twice.
 
+    @param sidno              The sidno that will be used to retrieve GNO
     @param start              The first possible value for the GNO
     @param end                The last possible value for the GNO
 
@@ -523,8 +551,8 @@ class Certifier : public Certifier_interface {
     @retval -1                Error: GNOs exhausted for group UUID.
     @retval -2                Error: generated GNO is bigger than end.
   */
-  rpl_gno get_group_next_available_gtid_candidate(rpl_gno start,
-                                                  rpl_gno end) const;
+  rpl_gno get_next_available_gtid_candidate(rpl_sidno sidno, rpl_gno start,
+                                            rpl_gno end) const;
 
   bool inline is_initialized() { return initialized; }
 
@@ -551,7 +579,7 @@ class Certifier : public Certifier_interface {
   int64 parallel_applier_last_committed_global;
   int64 parallel_applier_sequence_number;
 
-#if !defined(DBUG_OFF)
+#if !defined(NDEBUG)
   bool certifier_garbage_collection_block;
   bool same_member_message_discarded;
 #endif
@@ -646,11 +674,6 @@ class Certifier : public Certifier_interface {
   std::map<std::string, Gtid_set::Interval> member_gtids;
   ulonglong gtids_assigned_in_blocks_counter;
 
-  /**
-    Last local known GTID
-  */
-  Gtid last_local_gtid;
-
   /**
     Conflict detection is performed when:
      1) group is on multi-master mode;
@@ -692,16 +715,6 @@ class Certifier : public Certifier_interface {
   */
   Gtid_set *get_certified_write_set_snapshot_version(const char *item);
 
-  /**
-    Computes intersection between all sets received, so that we
-    have the already applied transactions on all servers.
-
-    @return the operation status
-      @retval 0      OK
-      @retval !=0    Error
-  */
-  int stable_set_handle();
-
   /**
     Removes the intersection of the received transactions stable
     sets from certification database.
@@ -734,8 +747,11 @@ class Gtid_Executed_Message : public Plugin_gcs_message {
     // Length of the payload item: variable
     PIT_GTID_EXECUTED = 1,
 
+    // Length of the payload item: 8 bytes
+    PIT_SENT_TIMESTAMP = 2,
+
     // No valid type codes can appear after this one.
-    PIT_MAX = 2
+    PIT_MAX = 3
   };
 
   /**
@@ -752,6 +768,18 @@ class Gtid_Executed_Message : public Plugin_gcs_message {
    */
   void append_gtid_executed(uchar *gtid_data, size_t len);
 
+  /**
+    Return the time at which the message contained in the buffer was sent.
+    @see Metrics_handler::get_current_time()
+
+    @param[in] buffer            the buffer to decode from.
+    @param[in] length            the buffer length
+
+    @return the time on which the message was sent.
+  */
+  static uint64_t get_sent_timestamp(const unsigned char *buffer,
+                                     size_t length);
+
  protected:
   /*
    Implementation of the template methods of Gcs_plugin_message
