@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,7 +44,8 @@ Group_member_info::Group_member_info(
     Group_member_info::Group_member_role role_arg, bool in_single_primary_mode,
     bool has_enforces_update_everywhere_checks, uint member_weight_arg,
     uint lower_case_table_names_arg, bool default_table_encryption_arg,
-    const char *recovery_endpoints_arg, PSI_mutex_key psi_mutex_key_arg)
+    const char *recovery_endpoints_arg, const char *view_change_uuid_arg,
+    bool allow_single_leader, PSI_mutex_key psi_mutex_key_arg)
     : Plugin_gcs_message(CT_MEMBER_INFO_MESSAGE),
       hostname(hostname_arg),
       port(port_arg),
@@ -63,8 +64,12 @@ Group_member_info::Group_member_info(
       primary_election_running(false),
       recovery_endpoints(recovery_endpoints_arg ? recovery_endpoints_arg
                                                 : "DEFAULT"),
-#ifndef DBUG_OFF
+      m_view_change_uuid(view_change_uuid_arg ? view_change_uuid_arg
+                                              : "AUTOMATIC"),
+      m_allow_single_leader(allow_single_leader),
+#ifndef NDEBUG
       skip_encode_default_table_encryption(false),
+      m_skip_encode_view_change_uuid(false),
 #endif
       psi_mutex_key(psi_mutex_key_arg) {
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -101,8 +106,14 @@ Group_member_info::Group_member_info(Group_member_info &other)
       group_action_running(other.is_group_action_running()),
       primary_election_running(other.is_primary_election_running()),
       recovery_endpoints(other.get_recovery_endpoints()),
-#ifndef DBUG_OFF
+      m_view_change_uuid(other.get_view_change_uuid()),
+      m_allow_single_leader(other.get_allow_single_leader()),
+      m_group_action_running_name(other.get_group_action_running_name()),
+      m_group_action_running_description(
+          other.get_group_action_running_description()),
+#ifndef NDEBUG
       skip_encode_default_table_encryption(false),
+      m_skip_encode_view_change_uuid(false),
 #endif
       psi_mutex_key(other.psi_mutex_key) {
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -122,8 +133,11 @@ Group_member_info::Group_member_info(const uchar *data, size_t len,
       group_action_running(false),
       primary_election_running(false),
       recovery_endpoints("DEFAULT"),
-#ifndef DBUG_OFF
+      m_view_change_uuid("AUTOMATIC"),
+      m_allow_single_leader(false),
+#ifndef NDEBUG
       skip_encode_default_table_encryption(false),
+      m_skip_encode_view_change_uuid(false),
 #endif
       psi_mutex_key(psi_mutex_key_arg) {
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -146,7 +160,8 @@ void Group_member_info::update(
     Group_member_info::Group_member_role role_arg, bool in_single_primary_mode,
     bool has_enforces_update_everywhere_checks, uint member_weight_arg,
     uint lower_case_table_names_arg, bool default_table_encryption_arg,
-    const char *recovery_endpoints_arg) {
+    const char *recovery_endpoints_arg, const char *view_change_uuid_arg,
+    bool allow_single_leader) {
   MUTEX_LOCK(lock, &update_lock);
 
   hostname.assign(hostname_arg);
@@ -181,6 +196,9 @@ void Group_member_info::update(
     configuration_flags |= CNF_ENFORCE_UPDATE_EVERYWHERE_CHECKS_F;
 
   recovery_endpoints.assign(recovery_endpoints_arg);
+
+  m_view_change_uuid.assign(view_change_uuid_arg);
+  m_allow_single_leader = allow_single_leader;
 }
 
 void Group_member_info::update(Group_member_info &other) {
@@ -196,7 +214,8 @@ void Group_member_info::update(Group_member_info &other) {
       other.get_configuration_flags() | CNF_ENFORCE_UPDATE_EVERYWHERE_CHECKS_F,
       other.get_member_weight(), other.get_lower_case_table_names(),
       other.get_default_table_encryption(),
-      other.get_recovery_endpoints().c_str());
+      other.get_recovery_endpoints().c_str(),
+      other.get_view_change_uuid().c_str(), other.get_allow_single_leader());
 }
 
 /*
@@ -261,7 +280,7 @@ void Group_member_info::encode_payload(
 
   uint16 lower_case_table_names_aux =
       static_cast<uint16>(lower_case_table_names);
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   if (lower_case_table_names != SKIP_ENCODING_LOWER_CASE_TABLE_NAMES)
 #endif
     encode_payload_item_int2(buffer, PIT_LOWER_CASE_TABLE_NAME,
@@ -280,7 +299,7 @@ void Group_member_info::encode_payload(
                            is_election_running_aux);
 
   char default_table_encryption_aux = default_table_encryption ? '1' : '0';
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   if (!skip_encode_default_table_encryption)
 #endif
     encode_payload_item_char(buffer, PIT_DEFAULT_TABLE_ENCRYPTION,
@@ -292,6 +311,26 @@ void Group_member_info::encode_payload(
   encode_payload_item_string(buffer, PIT_RECOVERY_ENDPOINTS,
                              recovery_endpoints.c_str(),
                              recovery_endpoints.length());
+
+#ifndef NDEBUG
+  if (!m_skip_encode_view_change_uuid)
+#endif
+    encode_payload_item_string(buffer, PIT_VIEW_CHANGE_UUID,
+                               m_view_change_uuid.c_str(),
+                               m_view_change_uuid.length());
+
+  char allow_single_leader_aux = m_allow_single_leader ? '1' : '0';
+  encode_payload_item_char(buffer, PIT_ALLOW_SINGLE_LEADER,
+                           allow_single_leader_aux);
+
+  if (group_action_running) {
+    encode_payload_item_string(buffer, PIT_GROUP_ACTION_RUNNING_NAME,
+                               m_group_action_running_name.c_str(),
+                               m_group_action_running_name.length());
+    encode_payload_item_string(buffer, PIT_GROUP_ACTION_RUNNING_DESCRIPTION,
+                               m_group_action_running_description.c_str(),
+                               m_group_action_running_description.length());
+  }
 }
 
 void Group_member_info::decode_payload(const unsigned char *buffer,
@@ -364,7 +403,6 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_CONFLICT_DETECTION_ENABLE:
         if (slider + payload_item_length <= end) {
           unsigned char conflict_detection_enable_aux = *slider;
-          slider += payload_item_length;
           conflict_detection_enable =
               (conflict_detection_enable_aux == '1') ? true : false;
         }
@@ -373,7 +411,6 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_MEMBER_WEIGHT:
         if (slider + payload_item_length <= end) {
           uint16 member_weight_aux = uint2korr(slider);
-          slider += payload_item_length;
           member_weight = (uint)member_weight_aux;
         }
         break;
@@ -381,7 +418,6 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_LOWER_CASE_TABLE_NAME:
         if (slider + payload_item_length <= end) {
           uint16 lower_case_table_names_aux = uint2korr(slider);
-          slider += payload_item_length;
           lower_case_table_names =
               static_cast<uint>(lower_case_table_names_aux);
         }
@@ -390,7 +426,6 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_GROUP_ACTION_RUNNING:
         if (slider + payload_item_length <= end) {
           unsigned char is_action_running_aux = *slider;
-          slider += payload_item_length;
           group_action_running = (is_action_running_aux == '1') ? true : false;
         }
         break;
@@ -398,7 +433,6 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_PRIMARY_ELECTION_RUNNING:
         if (slider + payload_item_length <= end) {
           unsigned char is_election_running_aux = *slider;
-          slider += payload_item_length;
           primary_election_running =
               (is_election_running_aux == '1') ? true : false;
         }
@@ -413,7 +447,6 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
       case PIT_DEFAULT_TABLE_ENCRYPTION:
         if (slider + payload_item_length <= end) {
           unsigned char default_table_encryption_aux = *slider;
-          slider += payload_item_length;
           default_table_encryption =
               (default_table_encryption_aux == '1') ? true : false;
         }
@@ -422,17 +455,46 @@ void Group_member_info::decode_payload(const unsigned char *buffer,
         if (slider + payload_item_length <= end) {
           purged_gtid_set.assign(reinterpret_cast<const char *>(slider),
                                  static_cast<size_t>(payload_item_length));
-          slider += payload_item_length;
         }
         break;
       case PIT_RECOVERY_ENDPOINTS:
         if (slider + payload_item_length <= end) {
           recovery_endpoints.assign(reinterpret_cast<const char *>(slider),
                                     static_cast<size_t>(payload_item_length));
-          slider += payload_item_length;
+        }
+        break;
+      case PIT_VIEW_CHANGE_UUID:
+        if (slider + payload_item_length <= end) {
+          m_view_change_uuid.assign(reinterpret_cast<const char *>(slider),
+                                    static_cast<size_t>(payload_item_length));
+        }
+        break;
+
+      case PIT_ALLOW_SINGLE_LEADER:
+        if (slider + payload_item_length <= end) {
+          unsigned char allow_single_leader_aux = *slider;
+          m_allow_single_leader =
+              (allow_single_leader_aux == '1') ? true : false;
+        }
+        break;
+      case PIT_GROUP_ACTION_RUNNING_NAME:
+        if (slider + payload_item_length <= end) {
+          m_group_action_running_name.assign(
+              reinterpret_cast<const char *>(slider),
+              static_cast<size_t>(payload_item_length));
+        }
+        break;
+      case PIT_GROUP_ACTION_RUNNING_DESCRIPTION:
+        if (slider + payload_item_length <= end) {
+          m_group_action_running_description.assign(
+              reinterpret_cast<const char *>(slider),
+              static_cast<size_t>(payload_item_length));
         }
         break;
     }
+
+    // Seek to next payload item.
+    slider += payload_item_length;
   }
 }
 
@@ -634,6 +696,24 @@ void Group_member_info::set_is_group_action_running(bool is_running) {
   group_action_running = is_running;
 }
 
+const std::string &Group_member_info::get_group_action_running_name() {
+  return m_group_action_running_name;
+}
+
+void Group_member_info::set_group_action_running_name(
+    const std::string &group_action_running_name) {
+  m_group_action_running_name = group_action_running_name;
+}
+
+const std::string &Group_member_info::get_group_action_running_description() {
+  return m_group_action_running_description;
+}
+
+void Group_member_info::set_group_action_running_description(
+    const std::string &group_action_running_description) {
+  m_group_action_running_description = group_action_running_description;
+}
+
 bool Group_member_info::is_primary_election_running() {
   MUTEX_LOCK(lock, &update_lock);
   return primary_election_running;
@@ -717,6 +797,21 @@ void Group_member_info::set_recovery_endpoints(const char *endpoints) {
   recovery_endpoints.assign(endpoints);
 }
 
+string Group_member_info::get_view_change_uuid() {
+  MUTEX_LOCK(lock, &update_lock);
+  return m_view_change_uuid;
+}
+
+bool Group_member_info::get_allow_single_leader() {
+  MUTEX_LOCK(lock, &update_lock);
+  return m_allow_single_leader;
+}
+
+void Group_member_info::set_view_change_uuid(const char *view_change_cnf) {
+  MUTEX_LOCK(lock, &update_lock);
+  m_view_change_uuid.assign(view_change_cnf);
+}
+
 bool Group_member_info::comparator_group_member_uuid(Group_member_info *m1,
                                                      Group_member_info *m2) {
   return m1->has_lower_uuid(m2);
@@ -755,7 +850,11 @@ bool Group_member_info::has_greater_weight(Group_member_info *other) {
 
 Group_member_info_manager::Group_member_info_manager(
     Group_member_info *local_member_info, PSI_mutex_key psi_mutex_key) {
-  members = new map<string, Group_member_info *>();
+  members = new std::map<
+      std::string, Group_member_info *, std::less<std::string>,
+      Malloc_allocator<std::pair<const std::string, Group_member_info *>>>(
+      Malloc_allocator<std::pair<const std::string, Group_member_info *>>(
+          key_group_member_info));
   this->local_member_info = local_member_info;
 
   mysql_mutex_init(psi_mutex_key, &update_lock, MY_MUTEX_INIT_FAST);
@@ -916,10 +1015,13 @@ Group_member_info_manager::get_group_member_status_by_member_id(
   return status;
 }
 
-vector<Group_member_info *> *Group_member_info_manager::get_all_members() {
+Group_member_info_list *Group_member_info_manager::get_all_members() {
   mysql_mutex_lock(&update_lock);
 
-  vector<Group_member_info *> *all_members = new vector<Group_member_info *>();
+  Group_member_info_list *all_members =
+      new std::vector<Group_member_info *,
+                      Malloc_allocator<Group_member_info *>>(
+          Malloc_allocator<Group_member_info *>(key_group_member_info));
   map<string, Group_member_info *>::iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *member_copy = new Group_member_info(*(*it).second);
@@ -978,13 +1080,12 @@ void Group_member_info_manager::update(Group_member_info *update_local_member) {
   mysql_mutex_unlock(&update_lock);
 }
 
-void Group_member_info_manager::update(
-    vector<Group_member_info *> *new_members) {
+void Group_member_info_manager::update(Group_member_info_list *new_members) {
   mysql_mutex_lock(&update_lock);
 
   this->clear_members();
 
-  vector<Group_member_info *>::iterator new_members_it;
+  Group_member_info_list_iterator new_members_it;
   for (new_members_it = new_members->begin();
        new_members_it != new_members->end(); new_members_it++) {
     // If this bears the local member to be updated
@@ -1172,9 +1273,9 @@ void Group_member_info_manager::encode(vector<uchar> *to_encode) {
   delete group_info_message;
 }
 
-vector<Group_member_info *> *Group_member_info_manager::decode(
+Group_member_info_list *Group_member_info_manager::decode(
     const uchar *to_decode, size_t length) {
-  vector<Group_member_info *> *decoded_members = nullptr;
+  Group_member_info_list *decoded_members = nullptr;
 
   Group_member_info_manager_message *group_info_message =
       new Group_member_info_manager_message();
@@ -1199,7 +1300,7 @@ bool Group_member_info_manager::get_primary_member_uuid(
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *info = (*it).second;
     if (info->get_role() == Group_member_info::MEMBER_ROLE_PRIMARY) {
-      DBUG_ASSERT(primary_member_uuid.empty());
+      assert(primary_member_uuid.empty());
       primary_member_uuid = info->get_uuid();
     }
   }
@@ -1226,7 +1327,7 @@ Group_member_info *Group_member_info_manager::get_primary_member_info() {
   }
   mysql_mutex_unlock(&update_lock);
 
-  DBUG_ASSERT(member_copy == nullptr || member_copy->in_primary_mode());
+  assert(member_copy == nullptr || member_copy->in_primary_mode());
 
   return member_copy;
 }
@@ -1305,7 +1406,9 @@ std::string Group_member_info_manager::get_string_current_view_active_hosts()
 Group_member_info_manager_message::Group_member_info_manager_message()
     : Plugin_gcs_message(CT_MEMBER_INFO_MANAGER_MESSAGE) {
   DBUG_TRACE;
-  members = new vector<Group_member_info *>();
+  members = new std::vector<Group_member_info *,
+                            Malloc_allocator<Group_member_info *>>(
+      Malloc_allocator<Group_member_info *>(key_group_member_info));
 }
 
 Group_member_info_manager_message::Group_member_info_manager_message(
@@ -1319,7 +1422,9 @@ Group_member_info_manager_message::Group_member_info_manager_message(
     Group_member_info *member_info)
     : Plugin_gcs_message(CT_MEMBER_INFO_MANAGER_MESSAGE), members(nullptr) {
   DBUG_TRACE;
-  members = new vector<Group_member_info *>();
+  members = new std::vector<Group_member_info *,
+                            Malloc_allocator<Group_member_info *>>(
+      Malloc_allocator<Group_member_info *>(key_group_member_info));
   members->push_back(member_info);
 }
 
@@ -1331,19 +1436,21 @@ Group_member_info_manager_message::~Group_member_info_manager_message() {
 
 void Group_member_info_manager_message::clear_members() {
   DBUG_TRACE;
-  std::vector<Group_member_info *>::iterator it;
+  Group_member_info_list_iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     delete (*it);
   }
   members->clear();
 }
 
-std::vector<Group_member_info *>
-    *Group_member_info_manager_message::get_all_members() {
+Group_member_info_list *Group_member_info_manager_message::get_all_members() {
   DBUG_TRACE;
-  vector<Group_member_info *> *all_members = new vector<Group_member_info *>();
+  Group_member_info_list *all_members =
+      new std::vector<Group_member_info *,
+                      Malloc_allocator<Group_member_info *>>(
+          Malloc_allocator<Group_member_info *>(key_group_member_info));
 
-  std::vector<Group_member_info *>::iterator it;
+  Group_member_info_list_iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     Group_member_info *member_copy = new Group_member_info(*(*it));
     all_members->push_back(member_copy);
@@ -1359,7 +1466,7 @@ void Group_member_info_manager_message::encode_payload(
   uint16 number_of_members = (uint16)members->size();
   encode_payload_item_int2(buffer, PIT_MEMBERS_NUMBER, number_of_members);
 
-  std::vector<Group_member_info *>::iterator it;
+  Group_member_info_list_iterator it;
   for (it = members->begin(); it != members->end(); it++) {
     std::vector<uchar> encoded_member;
     (*it)->encode(&encoded_member);
@@ -1397,3 +1504,71 @@ void Group_member_info_manager_message::decode_payload(
     slider += payload_item_length;
   }
 }
+
+void Group_member_info_manager_message::
+    add_member_actions_serialized_configuration(
+        std::vector<unsigned char> *buffer,
+        const std::string &member_actions_serialized_configuration) const {
+  DBUG_TRACE;
+
+  encode_payload_item_type_and_length(
+      buffer, PIT_MEMBER_ACTIONS,
+      member_actions_serialized_configuration.size());
+  buffer->insert(buffer->end(), member_actions_serialized_configuration.begin(),
+                 member_actions_serialized_configuration.end());
+}
+
+bool Group_member_info_manager_message::get_pit_data(
+    const enum_payload_item_type pit, const unsigned char *buffer,
+    size_t length, const unsigned char **pit_data, size_t *pit_length) {
+  DBUG_TRACE;
+  const unsigned char *slider = buffer;
+  const unsigned char *end = buffer + length;
+  uint16 payload_item_type = 0;
+  unsigned long long payload_item_length = 0;
+
+  decode_header(&slider);
+
+  uint16 number_of_members = 0;
+  decode_payload_item_int2(&slider, &payload_item_type, &number_of_members);
+
+  for (uint16 i = 0; i < number_of_members; i++) {
+    decode_payload_item_type_and_length(&slider, &payload_item_type,
+                                        &payload_item_length);
+    slider += payload_item_length;
+  }
+
+  while (slider + Plugin_gcs_message::WIRE_PAYLOAD_ITEM_HEADER_SIZE <= end) {
+    // Read payload item header to find payload item length.
+    decode_payload_item_type_and_length(&slider, &payload_item_type,
+                                        &payload_item_length);
+
+    if (pit == payload_item_type) {
+      if (slider + payload_item_length <= end) {
+        *pit_data = slider;
+        *pit_length = payload_item_length;
+        return false;
+      }
+    }
+
+    // Seek to next payload item.
+    slider += payload_item_length;
+  }
+
+  return true;
+}
+
+void Group_member_info_manager_message::
+    add_replication_failover_channels_serialized_configuration(
+        std::vector<unsigned char> *buffer,
+        const std::string
+            &replication_failover_channels_serialized_configuration) const {
+  DBUG_TRACE;
+
+  encode_payload_item_type_and_length(
+      buffer, PIT_RPL_FAILOVER_CONFIGURATION,
+      replication_failover_channels_serialized_configuration.size());
+  buffer->insert(buffer->end(),
+                 replication_failover_channels_serialized_configuration.begin(),
+                 replication_failover_channels_serialized_configuration.end());
+}
