@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,6 +32,7 @@
 #include <mysql/components/services/log_builtins.h>
 #include <mysqld_error.h>
 
+#include "m_string.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
@@ -40,7 +41,9 @@
 #include "template_utils.h"
 #include "thr_cond.h"
 
-#define STRING_BUFFER 1024
+static constexpr int STRING_BUFFER = 1024 * 4;
+
+struct CHARSET_INFO;
 
 static const char *sep =
     "======================================================\n";
@@ -145,7 +148,7 @@ struct st_send_field_n {
   enum_field_types type;
 };
 
-const CHARSET_INFO *sql_resultcs = NULL;
+const CHARSET_INFO *sql_resultcs = nullptr;
 uint sql_num_meta_rows = 0;
 uint sql_num_rows = 0;
 uint col_count = 0;
@@ -160,7 +163,7 @@ static int sql_start_result_metadata(void *, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->m_coll_name));
   row_count = 0;
   sql_num_cols = num_cols;
   sql_resultcs = resultcs;
@@ -325,16 +328,17 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
+    nullptr,
 };
 
 /****************************************************************************************/
 
-static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
+static void test_com_query(void *p [[maybe_unused]]) {
   DBUG_TRACE;
 
   /* Session declarations */
   MYSQL_SESSION st_session;
-  void *plugin_ctx = NULL;
+  void *plugin_ctx = nullptr;
   bool session_ret = false;
   bool fail = false;
   COM_DATA cmd;
@@ -353,11 +357,12 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
       "-----------------------------------------------------------------\n");
   memset(&sql_str_value, 0, 64 * 64 * 256 * sizeof(char));
   memset(&sql_str_len, 0, 64 * 64 * sizeof(size_t));
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "SELECT id,info FROM information_schema.processlist";
   cmd.com_query.length = strlen(cmd.com_query.query);
   WRITE_VAL("%s\n", cmd.com_query.query);
   fail = command_service_run_command(st_session, COM_QUERY, &cmd,
-                                     &my_charset_utf8_general_ci, &sql_cbs,
+                                     &my_charset_utf8mb3_general_ci, &sql_cbs,
                                      CS_TEXT_REPRESENTATION, &cbd);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "sql_simple ret code: %d\n",
@@ -366,7 +371,7 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
     /* get values */
     WRITE_STR(
         "-----------------------------------------------------------------\n");
-    WRITE_VAL("%s\t\%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
+    WRITE_VAL("%s\t%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
     for (uint row = 0; row < sql_num_rows; row++) {
       for (uint col = 0; col < sql_num_cols; col++) {
         WRITE_VAL("%s\n", sql_str_value[col][row]);
@@ -381,7 +386,7 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
     } else {
       WRITE_VAL("server status: %d\n", cbd.server_status);
       WRITE_VAL("warn count: %d\n", cbd.warn_count);
-      //           WRITE_VAL("messsage: %s\n",msg);
+      //           WRITE_VAL("message: %s\n",msg);
     }
   }
 
@@ -390,6 +395,7 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
       "-----------------------------------------------------------------\n");
   memset(&sql_str_value, 0, 64 * 64 * 256 * sizeof(char));
   memset(&sql_str_len, 0, 64 * 64 * sizeof(size_t));
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query =
       "SELECT * FROM performance_schema.global_variables WHERE variable_name "
       "LIKE 'INNODB_READ_IO_THREADS'";
@@ -397,7 +403,7 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
   WRITE_VAL("%s\n", cmd.com_query.query);
   cbd.reset();
   fail = command_service_run_command(st_session, COM_QUERY, &cmd,
-                                     &my_charset_utf8_general_ci, &sql_cbs,
+                                     &my_charset_utf8mb3_general_ci, &sql_cbs,
                                      CS_TEXT_REPRESENTATION, &cbd);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "sql_simple ret code: %d\n",
@@ -406,10 +412,10 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
     /* get values */
     WRITE_STR(
         "-----------------------------------------------------------------\n");
-    WRITE_VAL("%s\t\%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
+    WRITE_VAL("%s\t%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
     for (uint row = 0; row < sql_num_rows; row++) {
       for (uint col = 0; col < sql_num_cols; col += 2) {
-        WRITE_VAL("%s\t\%s\n", sql_str_value[col][row],
+        WRITE_VAL("%s\t%s\n", sql_str_value[col][row],
                   sql_str_value[col + 1][row]);
       }
     }
@@ -427,11 +433,12 @@ static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
 
   // 3. statement must fail
   cbd.reset();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "garbage";
   cmd.com_query.length = strlen(cmd.com_query.query);
 
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
+      st_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
 
   WRITE_VAL("error after bad SQL: %i: %s\n", cbd.err, cbd.errmsg.c_str());
@@ -460,7 +467,7 @@ static int test_com_init_db(void *p) {
   cmd.com_init_db.length = strlen("mysql");
   Callback_data cbd;
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_INIT_DB, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
+      st_session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
 
   db_name = srv_session_info_get_current_db(st_session);
@@ -485,7 +492,7 @@ static int test_com_list_fields(void *p)
   cmd.com_init_db.db_name = "mysql";
   cmd.com_init_db.length = strlen("mysql");
   ENSURE_API_OK(command_service_run_command(st_session, COM_INIT_DB, &cmd,
-&my_charset_utf8_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
+&my_charset_utf8mb3_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
 
   WRITE_VAL("switched default db to: %s\n",
 srv_session_info_get_current_db(st_session));
@@ -498,7 +505,7 @@ char*)cmd.com_field_list.table_name); cmd.com_field_list.query = (unsigned
 char*)"%"; cmd.com_field_list.query_length = strlen((const
 char*)cmd.com_field_list.query);
   ENSURE_API_OK(command_service_run_command(st_session, COM_FIELD_LIST, &cmd,
-&my_charset_utf8_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
+&my_charset_utf8mb3_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
 
   WRITE_STR("-----------------------------------------------------------------\n");
   for (uint row_count=0;row_count < sql_num_rows;row_count++){
@@ -524,7 +531,7 @@ struct Test_data {
   Test_data() {
     ready = 0;
     native_cond_init(&cond);
-    native_mutex_init(&mutex, NULL);
+    native_mutex_init(&mutex, nullptr);
   }
 
   ~Test_data() {
@@ -557,6 +564,7 @@ static void *test_session_thread(Test_data *tdata) {
   WRITE_VAL("session is dead? %i\n",
             thd_killed(srv_session_info_get_thd(tdata->session)));
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "select sleep(10)";
   cmd.com_query.length = strlen("select sleep(10)");
 
@@ -564,9 +572,9 @@ static void *test_session_thread(Test_data *tdata) {
 
   tdata->go();
 
-  int r = command_service_run_command(tdata->session, COM_QUERY, &cmd,
-                                      &my_charset_utf8_general_ci, &sql_cbs,
-                                      CS_TEXT_REPRESENTATION, &cbdata);
+  const int r = command_service_run_command(
+      tdata->session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
+      CS_TEXT_REPRESENTATION, &cbdata);
   WRITE_VAL("Killed run_command return value: %i\n", r);
 
   WRITE_VAL("thread shutdown: %i (%s)\n", cbdata.shutdown,
@@ -581,7 +589,7 @@ static void *test_session_thread(Test_data *tdata) {
 
   srv_session_deinit_thread();
 
-  return NULL;
+  return nullptr;
 }
 
 static void session_error_cb(void *, unsigned int sql_errno,
@@ -635,10 +643,11 @@ static int test_query_kill(void *p) {
   snprintf(buffer, sizeof(buffer), "kill query %i",
            srv_session_info_get_session_id(st_session_victim));
   WRITE_STR("run KILL QUERY\n");
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = buffer;
   cmd.com_query.length = strlen(buffer);
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
+      st_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
 
   void *ret;
@@ -673,8 +682,8 @@ static int test_com_process_kill(void *p) {
 
   cmd.com_kill.id = srv_session_info_get_session_id(st_session_victim);
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_PROCESS_KILL, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
-      CS_TEXT_REPRESENTATION, &cbd));
+      st_session, COM_PROCESS_KILL, &cmd, &my_charset_utf8mb3_general_ci,
+      &sql_cbs, CS_TEXT_REPRESENTATION, &cbd));
 
   WRITE_VAL("session is dead now? %i\n",
             thd_killed(srv_session_info_get_thd(st_session_victim)));
@@ -698,10 +707,11 @@ static int test_priv(void *p) {
 
   switch_user(root_session, user_privileged);
 
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "create user ordinary@localhost";
   cmd.com_query.length = strlen(cmd.com_query.query);
   ENSURE_API_OK(command_service_run_command(
-      root_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
+      root_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
   WRITE_VAL("create user as root: %i %s\n", cbd.err, cbd.errmsg.c_str());
 
@@ -712,10 +722,11 @@ static int test_priv(void *p) {
     switch_user(ordinary_session, user_ordinary);
 
     cbd.reset();
+    memset(&cmd, 0, sizeof(cmd));
     cmd.com_query.query = "create user bogus@localhost";
     cmd.com_query.length = strlen(cmd.com_query.query);
     ENSURE_API_OK(command_service_run_command(
-        ordinary_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci,
+        ordinary_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci,
         &sql_cbs, CS_TEXT_REPRESENTATION, &cbd));
 
     WRITE_VAL("create user supposed to fail: %i %s\n", cbd.err,
@@ -725,10 +736,11 @@ static int test_priv(void *p) {
   }
 
   cbd.reset();
+  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "drop user ordinary@localhost";
   cmd.com_query.length = strlen(cmd.com_query.query);
   ENSURE_API_OK(command_service_run_command(
-      root_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
+      root_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
   WRITE_VAL("drop user as root: %i %s\n", cbd.err, cbd.errmsg.c_str());
 
@@ -737,7 +749,7 @@ static int test_priv(void *p) {
   return 0;
 }
 
-static void test_sql(void *p MY_ATTRIBUTE((unused))) {
+static void test_sql(void *p [[maybe_unused]]) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Installation.");
 
@@ -780,7 +792,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
+static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
@@ -798,15 +810,15 @@ mysql_declare_plugin(test_daemon){
     MYSQL_DAEMON_PLUGIN,
     &test_sql_service_plugin,
     "test_sql_cmds_1",
-    "Horst Hunger, Andrey Hristov",
+    PLUGIN_AUTHOR_ORACLE,
     "Test sql service commands",
     PLUGIN_LICENSE_GPL,
     test_sql_service_plugin_init,   /* Plugin Init */
-    NULL,                           /* Plugin Check uninstall */
+    nullptr,                        /* Plugin Check uninstall */
     test_sql_service_plugin_deinit, /* Plugin Deinit */
     0x0100 /* 1.0 */,
-    NULL, /* status variables                */
-    NULL, /* system variables                */
-    NULL, /* config options                  */
-    0,    /* flags                           */
+    nullptr, /* status variables                */
+    nullptr, /* system variables                */
+    nullptr, /* config options                  */
+    0,       /* flags                           */
 } mysql_declare_plugin_end;
