@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -56,10 +56,10 @@
 #include "m_string.h"
 #include "my_dbug.h"
 #include "my_io.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_thread.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_thread.h"
 #include "mysqld_error.h"
 #include "sql-common/net_ns.h"
@@ -88,8 +88,11 @@ extern "C" {
 
 using std::max;
 
+// Test accept this many times
+static constexpr const uint MAX_ACCEPT_RETRY{10};
+
 /** Number of connection errors when selecting on the listening port */
-static std::atomic<ulong> connection_errors_select{0};
+static std::atomic<ulong> connection_errors_query_block{0};
 
 /** Number of connection errors when accepting sockets in the listening port. */
 static std::atomic<ulong> connection_errors_accept{0};
@@ -116,7 +119,9 @@ AddrInfoPtr GetAddrInfoPtr(const char *node, const char *service,
 }
 }  // namespace
 
-ulong get_connection_errors_select() { return connection_errors_select.load(); }
+ulong get_connection_errors_query_block() {
+  return connection_errors_query_block.load();
+}
 
 ulong get_connection_errors_accept() { return connection_errors_accept.load(); }
 
@@ -141,12 +146,14 @@ class Channel_info_local_socket : public Channel_info {
   MYSQL_SOCKET m_connect_sock;
 
  protected:
-  virtual Vio *create_and_init_vio() const {
+  Vio *create_and_init_vio() const override {
     Vio *vio =
         mysql_socket_vio_new(m_connect_sock, VIO_TYPE_SOCKET, VIO_LOCALHOST);
 #ifdef USE_PPOLL_IN_VIO
     if (vio != nullptr) {
-      vio->thread_id = my_thread_self();
+      // Unset thread_id, to ensure that all shutdowns explicitly set the
+      // current real_id from the THD.
+      vio->thread_id.reset();
       vio->signal_mask = mysqld_signal_mask;
     }
 #endif
@@ -162,18 +169,18 @@ class Channel_info_local_socket : public Channel_info {
   Channel_info_local_socket(MYSQL_SOCKET connect_socket)
       : m_connect_sock(connect_socket) {}
 
-  virtual THD *create_thd() {
+  THD *create_thd() override {
     THD *thd = Channel_info::create_thd();
 
-    if (thd != NULL) {
+    if (thd != nullptr) {
       init_net_server_extension(thd);
       thd->security_context()->set_host_ptr(my_localhost, strlen(my_localhost));
     }
     return thd;
   }
 
-  virtual void send_error_and_close_channel(uint errorcode, int error,
-                                            bool senderror) {
+  void send_error_and_close_channel(uint errorcode, int error,
+                                    bool senderror) override {
     Channel_info::send_error_and_close_channel(errorcode, error, senderror);
 
     mysql_socket_shutdown(m_connect_sock, SHUT_RDWR);
@@ -205,11 +212,11 @@ class Channel_info_tcpip_socket : public Channel_info {
 #endif
 
  protected:
-  virtual Vio *create_and_init_vio() const {
+  Vio *create_and_init_vio() const override {
     Vio *vio = mysql_socket_vio_new(m_connect_sock, VIO_TYPE_TCPIP, 0);
 #ifdef USE_PPOLL_IN_VIO
     if (vio != nullptr) {
-      vio->thread_id = my_thread_self();
+      vio->thread_id.reset();
       vio->signal_mask = mysqld_signal_mask;
     }
 #endif
@@ -234,25 +241,25 @@ class Channel_info_tcpip_socket : public Channel_info {
   Channel_info_tcpip_socket(MYSQL_SOCKET connect_socket, bool is_admin_conn)
       : m_connect_sock(connect_socket), m_is_admin_conn(is_admin_conn) {}
 
-  virtual THD *create_thd() {
+  THD *create_thd() override {
     THD *thd = Channel_info::create_thd();
 
-    if (thd != NULL) {
+    if (thd != nullptr) {
       thd->set_admin_connection(m_is_admin_conn);
       init_net_server_extension(thd);
     }
     return thd;
   }
 
-  virtual void send_error_and_close_channel(uint errorcode, int error,
-                                            bool senderror) {
+  void send_error_and_close_channel(uint errorcode, int error,
+                                    bool senderror) override {
     Channel_info::send_error_and_close_channel(errorcode, error, senderror);
 
     mysql_socket_shutdown(m_connect_sock, SHUT_RDWR);
     mysql_socket_close(m_connect_sock);
   }
 
-  virtual bool is_admin_connection() const { return m_is_admin_conn; }
+  bool is_admin_connection() const override { return m_is_admin_conn; }
 
 #ifdef HAVE_SETNS
   /**
@@ -300,7 +307,7 @@ class TCP_socket {
 
   MYSQL_SOCKET create_socket(const struct addrinfo *addrinfo_list,
                              int addr_family, struct addrinfo **use_addrinfo) {
-    for (const struct addrinfo *cur_ai = addrinfo_list; cur_ai != NULL;
+    for (const struct addrinfo *cur_ai = addrinfo_list; cur_ai != nullptr;
          cur_ai = cur_ai->ai_next) {
       if (cur_ai->ai_family != addr_family) continue;
 
@@ -310,7 +317,7 @@ class TCP_socket {
 
       char ip_addr[INET6_ADDRSTRLEN];
 
-      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), NULL, 0,
+      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), nullptr, 0,
                           NI_NUMERICHOST)) {
         ip_addr[0] = 0;
       }
@@ -356,7 +363,7 @@ class TCP_socket {
     @retval   valid socket if successful else MYSQL_INVALID_SOCKET on failure.
   */
   MYSQL_SOCKET get_listener_socket() {
-    const char *bind_address_str = NULL;
+    const char *bind_address_str = nullptr;
 
     LogErr(INFORMATION_LEVEL, ER_CONN_TCP_ADDRESS, m_bind_addr_str.c_str(),
            m_tcp_port);
@@ -403,7 +410,7 @@ class TCP_socket {
           dummy IPv6-socket. Do not instrument that socket by P_S.
         */
 
-        MYSQL_SOCKET s = mysql_socket_socket(0, AF_INET6, SOCK_STREAM, 0);
+        const MYSQL_SOCKET s = mysql_socket_socket(0, AF_INET6, SOCK_STREAM, 0);
         ipv6_available = mysql_socket_getfd(s) != INVALID_SOCKET;
         if (ipv6_available) mysql_socket_close(s);
       }
@@ -466,7 +473,7 @@ class TCP_socket {
          cur_ai = cur_ai->ai_next) {
       char ip_addr[INET6_ADDRSTRLEN];
 
-      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), NULL, 0,
+      if (vio_getnameinfo(cur_ai->ai_addr, ip_addr, sizeof(ip_addr), nullptr, 0,
                           NI_NUMERICHOST)) {
         LogErr(ERROR_LEVEL, ER_CONN_TCP_IP_NOT_LOGGED);
         continue;
@@ -614,12 +621,12 @@ class TCP_socket {
 */
 class Unix_socket {
   std::string m_unix_sockname;  // pathname for socket to bind to.
-  uint m_backlog;  // backlog specifying lenght of pending queue connection.
+  uint m_backlog;  // backlog specifying length of pending queue connection.
   /**
     Create a lockfile which contains the pid of the mysqld instance started
     and pathname as name of unix socket pathname appended with .lock
 
-    @retval   false if lockfile creation is successful else true if lockfile
+    @retval   False if lockfile creation is successful else true if lockfile
               file could not be created.
 
   */
@@ -654,7 +661,9 @@ class Unix_socket {
       return MYSQL_INVALID_SOCKET;
     }
 
-    if (create_lockfile()) {
+    if (create_lockfile() ||
+        DBUG_EVALUATE_IF("simulate_create_socket_lockfile_error", true,
+                         false)) {
       LogErr(ERROR_LEVEL, ER_CONN_UNIX_LOCK_FILE_FAIL);
       return MYSQL_INVALID_SOCKET;
     }
@@ -876,15 +885,16 @@ void Mysqld_socket_listener::add_socket_to_listener(
 }
 
 void Mysqld_socket_listener::setup_connection_events(
-    const socket_map_t &socket_map) {
+    const socket_vector_t &socket_vector) {
 #ifdef HAVE_POLL
-  const socket_map_t::size_type total_number_of_addresses_to_bind =
-      socket_map.size();
+  const socket_vector_t::size_type total_number_of_addresses_to_bind =
+      socket_vector.size();
   m_poll_info.m_fds.reserve(total_number_of_addresses_to_bind);
   m_poll_info.m_pfs_fds.reserve(total_number_of_addresses_to_bind);
 #endif
 
-  for (const auto &element : socket_map) add_socket_to_listener(element.first);
+  for (const auto &socket_element : socket_vector)
+    add_socket_to_listener(socket_element.m_socket);
 }
 
 /**
@@ -1063,15 +1073,15 @@ static bool handle_admin_socket(
   fd_set client_fds;
   FD_ZERO(&client_fds);
   FD_SET(mysql_socket_getfd(admin_socket), &client_fds);
-  int max_used_connection = mysql_socket_getfd(admin_socket);
+  const int max_used_connection = mysql_socket_getfd(admin_socket);
 #endif
 
   while (!connection_events_loop_aborted()) {
 #ifdef HAVE_POLL
-    int retval = poll(fds, NUMBER_OF_POLLED_FDS, -1);
+    const int retval = poll(fds, NUMBER_OF_POLLED_FDS, -1);
 #else
     fd_set read_fds = client_fds;
-    int retval = select(max_used_connection, &read_fds, 0, 0, 0);
+    const int retval = select(max_used_connection, &read_fds, 0, 0, 0);
 #endif
 
     if (retval < 0 && socket_errno != SOCKET_EINTR) {
@@ -1080,7 +1090,7 @@ static bool handle_admin_socket(
         There is not much details to report about the client,
         increment the server global status variable.
       */
-      ++connection_errors_select;
+      ++connection_errors_query_block;
       if (!select_errors++ && !connection_events_loop_aborted())
         LogErr(ERROR_LEVEL, ER_CONN_SOCKET_SELECT_FAILED, socket_errno);
     }
@@ -1104,7 +1114,7 @@ static bool handle_admin_socket(
 #endif
 
     MYSQL_SOCKET connect_sock;
-    bool accept_retval = accept_connection(admin_socket, &connect_sock);
+    const bool accept_retval = accept_connection(admin_socket, &connect_sock);
 
 #ifdef HAVE_SETNS
     if (!network_namespace_for_listening_socket.empty() &&
@@ -1164,7 +1174,7 @@ extern "C" void *admin_socket_thread(void *arg) {
   );
 
   my_thread_end();
-  my_thread_exit(0);
+  my_thread_exit(nullptr);
 
   return nullptr;
 }
@@ -1209,7 +1219,7 @@ static inline bool spawn_admin_thread(MYSQL_SOCKET admin_socket,
 
   if (arg_for_admin_socket_thread == nullptr) return true;
 
-  int ret = mysql_thread_create(
+  const int ret = mysql_thread_create(
       key_thread_handle_con_admin_sockets, &admin_socket_thread_id,
       &admin_socket_thread_attrib, admin_socket_thread,
       (void *)arg_for_admin_socket_thread);
@@ -1226,6 +1236,16 @@ static inline bool spawn_admin_thread(MYSQL_SOCKET admin_socket,
   return false;
 }
 
+bool Mysqld_socket_listener::check_and_spawn_admin_connection_handler_thread()
+    const {
+  if (m_use_separate_thread_for_admin) {
+    if (spawn_admin_thread(m_admin_interface_listen_socket,
+                           m_admin_bind_address.network_namespace))
+      return true;
+  }
+  return false;
+}
+
 bool Mysqld_socket_listener::setup_listener() {
   /*
     It's matter to add a socket for admin connection listener firstly,
@@ -1243,15 +1263,10 @@ bool Mysqld_socket_listener::setup_listener() {
 
     m_admin_interface_listen_socket = mysql_socket;
 
-    if (m_use_separate_thread_for_admin) {
-      if (spawn_admin_thread(m_admin_interface_listen_socket,
-                             m_admin_bind_address.network_namespace))
-        return true;
-    } else {
-      Socket_attr s(Socket_type::TCP_SOCKET,
-                    m_admin_bind_address.network_namespace);
-      m_socket_map.insert(
-          std::pair<MYSQL_SOCKET, Socket_attr>(mysql_socket, s));
+    if (!m_use_separate_thread_for_admin) {
+      m_socket_vector.emplace_back(mysql_socket, Socket_type::TCP_SOCKET,
+                                   &m_admin_bind_address.network_namespace,
+                                   Socket_interface_type::ADMIN_INTERFACE);
     }
   }
 
@@ -1264,11 +1279,9 @@ bool Mysqld_socket_listener::setup_listener() {
 
       MYSQL_SOCKET mysql_socket = tcp_socket.get_listener_socket();
       if (mysql_socket.fd == INVALID_SOCKET) return true;
-
-      Socket_attr s(Socket_type::TCP_SOCKET,
-                    bind_address_info.network_namespace);
-      m_socket_map.insert(
-          std::pair<MYSQL_SOCKET, Socket_attr>(mysql_socket, s));
+      m_socket_vector.emplace_back(mysql_socket, Socket_type::TCP_SOCKET,
+                                   &bind_address_info.network_namespace,
+                                   Socket_interface_type::DEFAULT_INTERFACE);
     }
   }
 #if defined(HAVE_SYS_UN_H)
@@ -1278,20 +1291,18 @@ bool Mysqld_socket_listener::setup_listener() {
 
     MYSQL_SOCKET mysql_socket = unix_socket.get_listener_socket();
     if (mysql_socket.fd == INVALID_SOCKET) return true;
-
-    m_socket_map.insert(std::pair<MYSQL_SOCKET, Socket_attr>(
-        mysql_socket, Socket_attr(Socket_type::UNIX_SOCKET)));
+    Listen_socket s(mysql_socket, Socket_type::UNIX_SOCKET);
+    m_socket_vector.push_back(s);
     m_unlink_sockname = true;
   }
 #endif /* HAVE_SYS_UN_H */
 
-  setup_connection_events(m_socket_map);
+  setup_connection_events(m_socket_vector);
 
   return false;
 }
 
-MYSQL_SOCKET Mysqld_socket_listener::get_ready_socket(
-    bool *is_unix_socket, bool *is_admin_socket) const {
+const Listen_socket *Mysqld_socket_listener::get_listen_socket() const {
 /*
   In case admin interface was set up, then first check whether an admin socket
   ready to accept a new connection. Doing this way provides higher priority
@@ -1299,24 +1310,17 @@ MYSQL_SOCKET Mysqld_socket_listener::get_ready_socket(
 */
 #ifdef HAVE_POLL
   uint start_index = 0;
-
   if (!m_admin_bind_address.address.empty() &&
       !m_use_separate_thread_for_admin) {
     if (m_poll_info.m_fds[0].revents & POLLIN) {
-      *is_unix_socket = false;
-      *is_admin_socket = true;
-      return m_admin_interface_listen_socket;
+      return &m_socket_vector[0];
     } else
       start_index = 1;
   }
 
-  *is_admin_socket = false;
-  for (uint i = start_index; i < m_socket_map.size(); ++i) {
+  for (uint i = start_index; i < m_socket_vector.size(); ++i) {
     if (m_poll_info.m_fds[i].revents & POLLIN) {
-      MYSQL_SOCKET listen_sock = m_poll_info.m_pfs_fds[i];
-      *is_unix_socket = m_socket_map.at(listen_sock).m_socket_type ==
-                        Socket_type::UNIX_SOCKET;
-      return listen_sock;
+      return &m_socket_vector[i];
     }
   }
 
@@ -1324,31 +1328,25 @@ MYSQL_SOCKET Mysqld_socket_listener::get_ready_socket(
   if (!m_admin_bind_address.address.empty() &&
       !m_use_separate_thread_for_admin &&
       FD_ISSET(mysql_socket_getfd(m_admin_interface_listen_socket),
-               &m_select_info.m_read_fds)) {
-    *is_unix_socket = false;
-    *is_admin_socket = true;
-    return m_admin_interface_listen_socket;
+               const_cast<fd_set *>(&m_select_info.m_read_fds))) {
+    return &m_socket_vector[0];
   }
 
-  *is_admin_socket = false;
-  for (socket_map_const_iterator_t sock_map_const_iter = m_socket_map.cbegin();
-       sock_map_const_iter != m_socket_map.cend(); ++sock_map_const_iter) {
-    if (FD_ISSET(mysql_socket_getfd(sock_map_const_iter->first),
-                 &m_select_info.m_read_fds)) {
-      *is_unix_socket =
-          sock_map_const_iter->second.m_socket_type == Socket_type::UNIX_SOCKET;
-      return sock_map_const_iter->first;
+  for (const auto &socket_element : m_socket_vector) {
+    if (FD_ISSET(mysql_socket_getfd(socket_element.m_socket),
+                 const_cast<fd_set *>(&m_select_info.m_read_fds))) {
+      return &socket_element;
     }
   }
 
 #endif  // HAVE_POLL
-
-  return MYSQL_INVALID_SOCKET;
+  return nullptr;
+  ;
 }
 
 Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
 #ifdef HAVE_POLL
-  int retval = poll(&m_poll_info.m_fds[0], m_socket_map.size(), -1);
+  int retval = poll(&m_poll_info.m_fds[0], m_socket_vector.size(), -1);
 #else
   m_select_info.m_read_fds = m_select_info.m_client_fds;
   int retval = select((int)m_select_info.m_max_used_connection,
@@ -1361,22 +1359,20 @@ Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
       There is not much details to report about the client,
       increment the server global status variable.
     */
-    ++connection_errors_select;
+    ++connection_errors_query_block;
     if (!select_errors++ && !connection_events_loop_aborted())
       LogErr(ERROR_LEVEL, ER_CONN_SOCKET_SELECT_FAILED, socket_errno);
   }
 
-  if (retval < 0 || connection_events_loop_aborted()) return NULL;
+  if (retval < 0 || connection_events_loop_aborted()) return nullptr;
 
   /* Is this a new connection request ? */
-  bool is_unix_socket = false, is_admin_sock;
-  MYSQL_SOCKET listen_sock = get_ready_socket(&is_unix_socket, &is_admin_sock);
+  const Listen_socket *listen_socket = get_listen_socket();
   /*
     When poll/select returns control flow then at least one ready server socket
     must exist. Check that get_ready_socket() returns a valid socket.
   */
-  DBUG_ASSERT(mysql_socket_getfd(listen_sock) != INVALID_SOCKET);
-
+  assert(listen_socket != nullptr);
   MYSQL_SOCKET connect_sock;
 #ifdef HAVE_SETNS
   /*
@@ -1388,15 +1384,17 @@ Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
     explicitly before calling accept().
   */
   std::string network_namespace_for_listening_socket;
-  if (!is_unix_socket) {
+  if (listen_socket->m_socket_type == Socket_type::TCP_SOCKET) {
     network_namespace_for_listening_socket =
-        m_socket_map.at(listen_sock).m_network_namespace;
+        (listen_socket->m_network_namespace != nullptr
+             ? *listen_socket->m_network_namespace
+             : std::string(""));
     if (!network_namespace_for_listening_socket.empty() &&
         set_network_namespace(network_namespace_for_listening_socket))
       return nullptr;
   }
 #endif
-  if (accept_connection(listen_sock, &connect_sock)) {
+  if (accept_connection(listen_socket->m_socket, &connect_sock)) {
 #ifdef HAVE_SETNS
     if (!network_namespace_for_listening_socket.empty())
       (void)restore_original_network_namespace();
@@ -1411,27 +1409,29 @@ Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
 #endif
 
 #ifdef HAVE_LIBWRAP
-  if (!is_unix_socket &&
+  if ((listen_socket->m_socket_type == Socket_type::TCP_SOCKET) &&
       check_connection_refused_by_tcp_wrapper(connect_sock)) {
     return nullptr;
   }
 #endif  // HAVE_LIBWRAP
 
-  Channel_info *channel_info = NULL;
-  if (is_unix_socket)
+  Channel_info *channel_info = nullptr;
+  if (listen_socket->m_socket_type == Socket_type::UNIX_SOCKET)
     channel_info = new (std::nothrow) Channel_info_local_socket(connect_sock);
   else
-    channel_info = new (std::nothrow)
-        Channel_info_tcpip_socket(connect_sock, is_admin_sock);
-  if (channel_info == NULL) {
+    channel_info = new (std::nothrow) Channel_info_tcpip_socket(
+        connect_sock, (listen_socket->m_socket_interface ==
+                       Socket_interface_type::ADMIN_INTERFACE));
+  if (channel_info == nullptr) {
     (void)mysql_socket_shutdown(connect_sock, SHUT_RDWR);
     (void)mysql_socket_close(connect_sock);
     connection_errors_internal++;
-    return NULL;
+    return nullptr;
   }
 
 #ifdef HAVE_SETNS
-  if (!is_unix_socket && !network_namespace_for_listening_socket.empty())
+  if (listen_socket->m_socket_type == Socket_type::TCP_SOCKET &&
+      !network_namespace_for_listening_socket.empty())
     static_cast<Channel_info_tcpip_socket *>(channel_info)
         ->set_network_namespace(network_namespace_for_listening_socket);
 #endif
@@ -1439,10 +1439,9 @@ Channel_info *Mysqld_socket_listener::listen_for_connection_event() {
 }
 
 void Mysqld_socket_listener::close_listener() {
-  for (socket_map_const_iterator_t sock_map_const_iter = m_socket_map.cbegin();
-       sock_map_const_iter != m_socket_map.cend(); ++sock_map_const_iter) {
-    (void)mysql_socket_shutdown(sock_map_const_iter->first, SHUT_RDWR);
-    (void)mysql_socket_close(sock_map_const_iter->first);
+  for (const auto &socket_element : m_socket_vector) {
+    (void)mysql_socket_shutdown(socket_element.m_socket, SHUT_RDWR);
+    (void)mysql_socket_close(socket_element.m_socket);
   }
 
   /*
@@ -1452,28 +1451,36 @@ void Mysqld_socket_listener::close_listener() {
     referenced by the data member m_admin_interface_listen_socket.
   */
   if (m_use_separate_thread_for_admin) {
+    if (admin_socket_thread_id != my_thread_handle{}) {
 #ifdef _WIN32
-    /*
-      For Windows, first close the socket referenced by the data member
-      m_admin_interface_listen_socket. It results in return from select()
-      API call running from a separate thread.
-    */
-    (void)mysql_socket_close(m_admin_interface_listen_socket);
-    my_thread_join(&admin_socket_thread_id, nullptr);
+      /*
+        For Windows, first close the socket referenced by the data member
+        m_admin_interface_listen_socket. It results in return from select()
+        API call running from a separate thread.
+      */
+      (void)mysql_socket_close(m_admin_interface_listen_socket);
+      m_admin_interface_listen_socket.fd = INVALID_SOCKET;
+      my_thread_join(&admin_socket_thread_id, nullptr);
 #else
-    // First, finish listening thread.
-    pthread_kill(admin_socket_thread_id.thread, SIGALRM);
-    my_thread_join(&admin_socket_thread_id, nullptr);
+      // First, finish listening thread.
+      pthread_kill(admin_socket_thread_id.thread, SIGALRM);
+      my_thread_join(&admin_socket_thread_id, nullptr);
+
+#endif
+      mysql_mutex_destroy(&LOCK_start_admin_thread);
+      mysql_cond_destroy(&COND_start_admin_thread);
+    }  // if (admin_socket_thread_id != my_thread_handle{})
     /*
       After a thread listening on admin interface finished, it is safe
-      to close listening socket.
+      to close listening socket. But socket is opened before thread is
+      spawned, so we may need to close the socket even if
+      admin_socket_thread_id has not been assigned.
     */
-    (void)mysql_socket_close(m_admin_interface_listen_socket);
-#endif
-
-    mysql_mutex_destroy(&LOCK_start_admin_thread);
-    mysql_cond_destroy(&COND_start_admin_thread);
-  }
+    if (m_admin_interface_listen_socket.fd != INVALID_SOCKET) {
+      (void)mysql_socket_close(m_admin_interface_listen_socket);
+      m_admin_interface_listen_socket.fd = INVALID_SOCKET;
+    }
+  }  // use_separate_thread_for_admin
 
 #if defined(HAVE_SYS_UN_H)
   if (m_unix_sockname != "" && m_unlink_sockname) {
@@ -1487,6 +1494,6 @@ void Mysqld_socket_listener::close_listener() {
   release_network_namespace_resources();
 #endif
 
-  m_socket_map.clear();
+  m_socket_vector.clear();
   m_bind_addresses.clear();
 }
