@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,6 +26,7 @@
 
 #include <string.h>
 #include <algorithm>
+#include <optional>
 #include <string>
 #include <type_traits>
 
@@ -38,16 +39,17 @@
 #include "my_compare.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_macros.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/plugin.h"
+#include "mysql/strings/dtoa.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
-#include "nullable.h"
 #include "sql/dd/collection.h"
 #include "sql/dd/dd_table.h"       // dd::FIELD_NAME_SEPARATOR_CHAR
 #include "sql/dd/dd_tablespace.h"  // dd::get_tablespace_name
@@ -70,6 +72,7 @@
 #include "sql/field.h"
 #include "sql/gis/srid.h"
 #include "sql/handler.h"
+#include "sql/histograms/table_histograms.h"
 #include "sql/key.h"
 #include "sql/log.h"
 #include "sql/partition_element.h"  // partition_element
@@ -90,10 +93,6 @@
 #include "sql/thd_raii.h"
 #include "typelib.h"
 
-namespace histograms {
-class Histogram;
-}  // namespace histograms
-
 enum_field_types dd_get_old_field_type(dd::enum_column_types type) {
   switch (type) {
     case dd::enum_column_types::DECIMAL:
@@ -190,7 +189,7 @@ enum_field_types dd_get_old_field_type(dd::enum_column_types type) {
       return MYSQL_TYPE_JSON;
 
     default:
-      DBUG_ASSERT(!"Should not hit here"); /* purecov: deadcode */
+      assert(!"Should not hit here"); /* purecov: deadcode */
   }
 
   return MYSQL_TYPE_LONG;
@@ -216,7 +215,7 @@ static enum ha_key_alg dd_get_old_index_algorithm_type(
       return HA_KEY_ALG_FULLTEXT;
 
     default:
-      DBUG_ASSERT(!"Should not hit here"); /* purecov: deadcode */
+      assert(!"Should not hit here"); /* purecov: deadcode */
   }
 
   return HA_KEY_ALG_SE_SPECIFIC;
@@ -234,7 +233,7 @@ bool is_suitable_for_primary_key(KEY_PART_INFO *key_part, Field *table_field) {
 
   /*
     If the key column is of NOT NULL BLOB type, then it
-    will definitly have key prefix. And if key part prefix size
+    will definitely have key prefix. And if key part prefix size
     is equal to the BLOB column max size, then we can promote
     it to primary key.
    */
@@ -260,33 +259,22 @@ bool is_suitable_for_primary_key(KEY_PART_INFO *key_part, Field *table_field) {
 static bool prepare_share(THD *thd, TABLE_SHARE *share,
                           const dd::Table *table_def) {
   my_bitmap_map *bitmaps;
-  bool use_hash;
   handler *handler_file = nullptr;
 
   // Mark 'system' tables (tables with one row) to help the Optimizer.
   share->system =
       ((share->max_rows == 1) && (share->min_rows == 1) && (share->keys == 0));
 
-  bool use_extended_sk = ha_check_storage_engine_flag(
+  const bool use_extended_sk = ha_check_storage_engine_flag(
       share->db_type(), HTON_SUPPORTS_EXTENDED_KEYS);
-  // Setup name_hash for quick look-up
-  use_hash = share->fields >= MAX_FIELDS_BEFORE_HASH;
-  if (use_hash) {
-    Field **field_ptr = share->field;
-    share->name_hash = new collation_unordered_map<std::string, Field **>(
-        system_charset_info, PSI_INSTRUMENT_ME);
-    share->name_hash->reserve(share->fields);
-
-    for (uint i = 0; i < share->fields; i++, field_ptr++) {
-      share->name_hash->emplace((*field_ptr)->field_name, field_ptr);
-    }
-  }
-
-  share->m_histograms =
-      new malloc_unordered_map<uint, const histograms::Histogram *>(
-          PSI_INSTRUMENT_ME);
 
   // Setup other fields =====================================================
+
+  if (share->tmp_table == NO_TMP_TABLE) {
+    share->m_histograms = new (&share->mem_root) Table_histograms_collection();
+    if (share->m_histograms == nullptr) return true;  // OOM.
+  }
+
   /* Allocate handler */
   if (!(handler_file = get_new_handler(share, (share->m_part_info != nullptr),
                                        &share->mem_root, share->db_type()))) {
@@ -307,7 +295,7 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
     KEY_PART_INFO *key_part;
     uint primary_key = (uint)(
         find_type(primary_key_name, &share->keynames, FIND_TYPE_NO_PREFIX) - 1);
-    longlong ha_option = handler_file->ha_table_flags();
+    const longlong ha_option = handler_file->ha_table_flags();
     keyinfo = share->key_info;
     key_part = keyinfo->key_part;
 
@@ -328,10 +316,10 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
       keyinfo->name = share->keynames.type_names[key];
 
       /* Check that fulltext and spatial keys have correct algorithm set. */
-      DBUG_ASSERT(!(share->key_info[key].flags & HA_FULLTEXT) ||
-                  share->key_info[key].algorithm == HA_KEY_ALG_FULLTEXT);
-      DBUG_ASSERT(!(share->key_info[key].flags & HA_SPATIAL) ||
-                  share->key_info[key].algorithm == HA_KEY_ALG_RTREE);
+      assert(!(share->key_info[key].flags & HA_FULLTEXT) ||
+             share->key_info[key].algorithm == HA_KEY_ALG_FULLTEXT);
+      assert(!(share->key_info[key].flags & HA_SPATIAL) ||
+             share->key_info[key].algorithm == HA_KEY_ALG_RTREE);
 
       if (primary_key >= MAX_KEY && (keyinfo->flags & HA_NOSAME)) {
         /*
@@ -352,7 +340,7 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
           Check that dd::Index::is_candidate_key() used by SEs works in
           the same way as above call to is_suitable_for_primary_key().
         */
-        DBUG_ASSERT((primary_key == key) == (*idx_it)->is_candidate_key());
+        assert((primary_key == key) == (*idx_it)->is_candidate_key());
       }
 
       dd::Index::Index_elements::const_iterator idx_el_it(
@@ -440,9 +428,8 @@ static bool prepare_share(THD *thd, TABLE_SHARE *share,
           Check that dd::Index_element::is_prefix() used by SEs works in
           the same way as code which sets HA_PART_KEY_SEG flag.
         */
-        DBUG_ASSERT(
-            (*idx_el_it)->is_prefix() ==
-            static_cast<bool>(key_part->key_part_flag & HA_PART_KEY_SEG));
+        assert((*idx_el_it)->is_prefix() ==
+               static_cast<bool>(key_part->key_part_flag & HA_PART_KEY_SEG));
         ++idx_el_it;
       }
 
@@ -566,7 +553,7 @@ static row_type dd_get_old_row_format(dd::Table::enum_row_format new_format) {
     case dd::Table::RF_PAGED:
       return ROW_TYPE_PAGED;
     default:
-      DBUG_ASSERT(0);
+      assert(0);
       break;
   }
   return ROW_TYPE_FIXED;
@@ -584,7 +571,7 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
   } else {
     // If no secondary storage engine is set, the share cannot
     // represent a table in a secondary engine.
-    DBUG_ASSERT(!share->is_secondary_engine());
+    assert(!share->is_secondary_engine());
   }
 
   // Read table engine type
@@ -594,12 +581,12 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
 
   plugin_ref tmp_plugin = ha_resolve_by_name_raw(thd, engine_name);
   if (tmp_plugin) {
-#ifndef DBUG_OFF
+#ifndef NDEBUG
     handlerton *hton = plugin_data<handlerton *>(tmp_plugin);
 #endif
 
-    DBUG_ASSERT(hton && ha_storage_engine_is_enabled(hton));
-    DBUG_ASSERT(!ha_check_storage_engine_flag(hton, HTON_NOT_USER_SELECTABLE));
+    assert(hton && ha_storage_engine_is_enabled(hton));
+    assert(!ha_check_storage_engine_flag(hton, HTON_NOT_USER_SELECTABLE));
 
     plugin_unlock(nullptr, share->db_plugin);
     share->db_plugin = my_plugin_lock(nullptr, &tmp_plugin);
@@ -609,7 +596,7 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
   }
 
   // Set temporarily a good value for db_low_byte_first.
-  DBUG_ASSERT(ha_legacy_type(share->db_type()) != DB_TYPE_ISAM);
+  assert(ha_legacy_type(share->db_type()) != DB_TYPE_ISAM);
   share->db_low_byte_first = true;
 
   // Read other table options
@@ -721,7 +708,7 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
   if (fill_tablespace_from_dd(thd, share, tab_obj)) return true;
 
   // Read comment
-  dd::String_type comment = tab_obj->comment();
+  const dd::String_type comment = tab_obj->comment();
   if (comment.length()) {
     share->comment.str =
         strmake_root(&share->mem_root, comment.c_str(), comment.length() + 1);
@@ -747,6 +734,10 @@ static bool fill_share_from_dd(THD *thd, TABLE_SHARE *share,
   if (table_options.exists("encrypt_type"))
     table_options.get("encrypt_type", &share->encrypt_type, &share->mem_root);
 
+  // Read secondary load option.
+  if (table_options.exists("secondary_load"))
+    table_options.get("secondary_load", &share->secondary_load);
+
   return false;
 }
 
@@ -784,12 +775,11 @@ inline void get_auto_flags(const dd::Column &col_obj, uint &auto_flags) {
       // "CURRENT_TIMESTAP(<integer>)". Stored functions with names
       // starting with "CURRENT_TIMESTAMP" should be filtered out before
       // we get here.
-      DBUG_ASSERT(
-          col_obj.default_option().size() == 17 ||
-          (col_obj.default_option().size() >= 20 &&
-           col_obj.default_option()[17] == '(' &&
-           col_obj.default_option()[col_obj.default_option().size() - 1] ==
-               ')'));
+      assert(col_obj.default_option().size() == 17 ||
+             (col_obj.default_option().size() >= 20 &&
+              col_obj.default_option()[17] == '(' &&
+              col_obj.default_option()[col_obj.default_option().size() - 1] ==
+                  ')'));
 
       auto_flags |= Field::DEFAULT_NOW;
     } else {
@@ -809,9 +799,9 @@ inline void get_auto_flags(const dd::Column &col_obj, uint &auto_flags) {
     Columns can't have AUTO_INCREMENT and DEFAULT/ON UPDATE CURRENT_TIMESTAMP at
     the same time.
   */
-  DBUG_ASSERT(!((auto_flags & (Field::DEFAULT_NOW | Field::ON_UPDATE_NOW |
-                               Field::GENERATED_FROM_EXPRESSION)) != 0 &&
-                (auto_flags & Field::NEXT_NUMBER) != 0));
+  assert(!((auto_flags & (Field::DEFAULT_NOW | Field::ON_UPDATE_NOW |
+                          Field::GENERATED_FROM_EXPRESSION)) != 0 &&
+           (auto_flags & Field::NEXT_NUMBER) != 0));
 }
 
 static Field *make_field(const dd::Column &col_obj, const CHARSET_INFO *charset,
@@ -833,7 +823,7 @@ static Field *make_field(const dd::Column &col_obj, const CHARSET_INFO *charset,
     //
     // Allocate space for interval (column elements)
     //
-    size_t interval_parts = col_obj.elements_count();
+    const size_t interval_parts = col_obj.elements_count();
 
     interval = (TYPELIB *)share->mem_root.Alloc(sizeof(TYPELIB));
     interval->type_names = (const char **)share->mem_root.Alloc(
@@ -861,15 +851,15 @@ static Field *make_field(const dd::Column &col_obj, const CHARSET_INFO *charset,
 
   // Column name
   char *name = nullptr;
-  dd::String_type s = col_obj.name();
-  DBUG_ASSERT(!s.empty());
+  const dd::String_type s = col_obj.name();
+  assert(!s.empty());
   name = strmake_root(&share->mem_root, s.c_str(), s.length());
   name[s.length()] = '\0';
 
   uint decimals;
   // Decimals
   if (field_type == MYSQL_TYPE_DECIMAL || field_type == MYSQL_TYPE_NEWDECIMAL) {
-    DBUG_ASSERT(col_obj.is_numeric_scale_null() == false);
+    assert(col_obj.is_numeric_scale_null() == false);
     decimals = col_obj.numeric_scale();
   } else if (field_type == MYSQL_TYPE_FLOAT ||
              field_type == MYSQL_TYPE_DOUBLE) {
@@ -919,8 +909,8 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
   //
 
   // Column name
-  dd::String_type s = col_obj->name();
-  DBUG_ASSERT(!s.empty());
+  const dd::String_type s = col_obj->name();
+  assert(!s.empty());
   name = strmake_root(&share->mem_root, s.c_str(), s.length());
   name[s.length()] = '\0';
 
@@ -949,7 +939,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
 
   // Decimals
   if (field_type == MYSQL_TYPE_DECIMAL || field_type == MYSQL_TYPE_NEWDECIMAL)
-    DBUG_ASSERT(col_obj->is_numeric_scale_null() == false);
+    assert(col_obj->is_numeric_scale_null() == false);
 
   // Read geometry sub type
   if (field_type == MYSQL_TYPE_GEOMETRY) {
@@ -979,7 +969,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
     //
     // Allocate space for interval (column elements)
     //
-    size_t interval_parts = col_obj->elements_count();
+    const size_t interval_parts = col_obj->elements_count();
 
     interval = (TYPELIB *)share->mem_root.Alloc(sizeof(TYPELIB));
     interval->type_names = (const char **)share->mem_root.Alloc(
@@ -1021,7 +1011,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
     gcol_info->set_field_stored(!col_obj->is_virtual());
 
     // Read generation expression.
-    dd::String_type gc_expr = col_obj->generation_expression();
+    const dd::String_type gc_expr = col_obj->generation_expression();
 
     /*
       Place the expression's text into the TABLE_SHARE. Field objects of
@@ -1045,7 +1035,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
     default_val_expr->set_field_stored(true);
 
     // Read generation expression.
-    dd::String_type default_val_expr_str = col_obj->default_option();
+    const dd::String_type default_val_expr_str = col_obj->default_option();
 
     // Copy the expression's text into reg_field which is stored on TABLE_SHARE.
     default_val_expr->dup_expr_str(&share->mem_root,
@@ -1087,7 +1077,7 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
   reg_field->set_column_format(field_column_format);
 
   // Comments
-  dd::String_type comment = col_obj->comment();
+  const dd::String_type comment = col_obj->comment();
   reg_field->comment.length = comment.length();
   if (reg_field->comment.length) {
     reg_field->comment.str =
@@ -1121,14 +1111,14 @@ static bool fill_column_from_dd(THD *thd, TABLE_SHARE *share,
 static bool fill_columns_from_dd(THD *thd, TABLE_SHARE *share,
                                  const dd::Table *tab_obj) {
   // Allocate space for fields in TABLE_SHARE.
-  uint fields_size = ((share->fields + 1) * sizeof(Field *));
+  const uint fields_size = ((share->fields + 1) * sizeof(Field *));
   share->field = (Field **)share->mem_root.Alloc((uint)fields_size);
   memset(share->field, 0, fields_size);
   share->vfields = 0;
   share->gen_def_field_count = 0;
 
   // Iterate through all the columns.
-  uchar *null_flags MY_ATTRIBUTE((unused));
+  uchar *null_flags [[maybe_unused]];
   uchar *null_pos, *rec_pos;
   null_flags = null_pos = share->default_values;
   rec_pos = share->default_values + share->null_bytes;
@@ -1207,10 +1197,9 @@ static bool fill_columns_from_dd(THD *thd, TABLE_SHARE *share,
   }
 
   // Make sure the scan of the columns is consistent with other data.
-  DBUG_ASSERT(share->null_bytes ==
-              (null_pos - null_flags + (null_bit_pos + 7) / 8));
-  DBUG_ASSERT(share->last_null_bit_pos == null_bit_pos);
-  DBUG_ASSERT(share->fields == field_nr);
+  assert(share->null_bytes == (null_pos - null_flags + (null_bit_pos + 7) / 8));
+  assert(share->last_null_bit_pos == null_bit_pos);
+  assert(share->fields == field_nr);
 
   return (false);
 }
@@ -1230,7 +1219,7 @@ static void fill_index_element_from_dd(TABLE_SHARE *share,
   keypart->fieldnr = idx_elem_obj->column().ordinal_position();
 
   // field
-  DBUG_ASSERT(keypart->fieldnr > 0);
+  assert(keypart->fieldnr > 0);
   Field *field = keypart->field = share->field[keypart->fieldnr - 1];
 
   // offset
@@ -1331,7 +1320,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
       keyinfo->flags = HA_NOSAME;
       break;
     default:
-      DBUG_ASSERT(0); /* purecov: deadcode */
+      assert(0); /* purecov: deadcode */
       keyinfo->flags = 0;
       break;
   }
@@ -1365,7 +1354,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   */
   uint32 stored_flags = 0;
   idx_options.get("flags", &stored_flags);
-  DBUG_ASSERT((stored_flags & ~(HA_PACK_KEY | HA_BINARY_PACK_KEY)) == 0);
+  assert((stored_flags & ~(HA_PACK_KEY | HA_BINARY_PACK_KEY)) == 0);
 
   //  Beginning in 8.0.12 HA_PACK_KEY and HA_BINARY_PACK_KEY are no longer set
   //  if the SE does not support it. If the index was created prior to 8.0.12
@@ -1391,7 +1380,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   if (idx_options.exists("block_size")) {
     idx_options.get("block_size", &keyinfo->block_size);
 
-    DBUG_ASSERT(keyinfo->block_size);
+    assert(keyinfo->block_size);
 
     keyinfo->flags |= HA_USES_BLOCK_SIZE;
   }
@@ -1400,7 +1389,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   if (idx_options.exists("parser_name")) {
     LEX_CSTRING parser_name;
     if (idx_options.get("parser_name", &parser_name, &share->mem_root))
-      DBUG_ASSERT(false);
+      assert(false);
 
     keyinfo->parser =
         my_plugin_lock_by_name(nullptr, parser_name, MYSQL_FTPARSER_PLUGIN);
@@ -1413,7 +1402,7 @@ static bool fill_index_from_dd(THD *thd, TABLE_SHARE *share,
   }
 
   // Read comment
-  dd::String_type comment = idx_obj->comment();
+  const dd::String_type comment = idx_obj->comment();
   keyinfo->comment.length = comment.length();
 
   if (keyinfo->comment.length) {
@@ -1479,12 +1468,12 @@ static bool fill_indexes_from_dd(THD *thd, TABLE_SHARE *share,
 
   uint32 primary_key_parts = 0;
 
-  bool use_extended_sk = ha_check_storage_engine_flag(
+  const bool use_extended_sk = ha_check_storage_engine_flag(
       share->db_type(), HTON_SUPPORTS_EXTENDED_KEYS);
 
   // Count number of keys and total number of key parts in the table.
 
-  DBUG_ASSERT(share->keys == 0 && share->key_parts == 0);
+  assert(share->keys == 0 && share->key_parts == 0);
 
   for (const dd::Index *idx_obj : tab_obj->indexes()) {
     // Skip hidden indexes
@@ -1529,18 +1518,15 @@ static bool fill_indexes_from_dd(THD *thd, TABLE_SHARE *share,
       return true; /* purecov: inspected */
 
     //
-    // Alloc buffer to hold keys and key_parts
+    // Alloc buffers to hold keys and key_parts
     //
 
-    if (!(share->key_info = (KEY *)share->mem_root.Alloc(
-              share->keys * sizeof(KEY) +
-              total_key_parts * sizeof(KEY_PART_INFO))))
+    if (!(share->key_info = share->mem_root.ArrayAlloc<KEY>(share->keys)))
       return true; /* purecov: inspected */
 
-    memset(
-        share->key_info, 0,
-        (share->keys * sizeof(KEY) + total_key_parts * sizeof(KEY_PART_INFO)));
-    key_part = (KEY_PART_INFO *)(share->key_info + share->keys);
+    if (!(key_part =
+              share->mem_root.ArrayAlloc<KEY_PART_INFO>(total_key_parts)))
+      return true; /* purecov: inspected */
 
     //
     // Alloc buffer to hold keynames
@@ -1555,8 +1541,8 @@ static bool fill_indexes_from_dd(THD *thd, TABLE_SHARE *share,
     share->keynames.count = share->keys;
 
     // In first iteration get all the index_obj, so that we get all
-    // user_defined_key_parts for each key. This is required to propertly
-    // allocation key_part memory for keys.
+    // user_defined_key_parts for each key. This is required to properly
+    // allocate key_part memory for keys.
     const dd::Index *index_at_pos[MAX_INDEXES];
     uint key_nr = 0;
     for (const dd::Index *idx_obj : tab_obj->indexes()) {
@@ -1650,7 +1636,7 @@ static void get_partition_options(MEM_ROOT *mem_root,
   if (part_options.exists("nodegroup_id"))
     part_options.get("nodegroup_id", &nodegroup_id);
 
-  DBUG_ASSERT(nodegroup_id <= 0xFFFF);
+  assert(nodegroup_id <= 0xFFFF);
   part_elem->nodegroup_id = nodegroup_id;
 }
 
@@ -1669,7 +1655,7 @@ static bool get_part_column_values(MEM_ROOT *mem_root,
     entries++;
   }
   if (entries != ((max_column_id + 1) * (max_list_index + 1))) {
-    DBUG_ASSERT(0); /* purecov: deadcode */
+    assert(0); /* purecov: deadcode */
     return true;
   }
 
@@ -1712,11 +1698,11 @@ static bool get_part_column_values(MEM_ROOT *mem_root,
 
   for (list_index = 0; list_index <= max_list_index; list_index++) {
     p_val = &p_elem_values[list_index];
-#ifndef DBUG_OFF
+#ifndef NDEBUG
     for (uint i = 0; i < part_info->num_columns; i++) {
-      DBUG_ASSERT(p_val->col_val_array[i].null_value ||
-                  p_val->col_val_array[i].max_value ||
-                  p_val->col_val_array[i].column_value.value_str);
+      assert(p_val->col_val_array[i].null_value ||
+             p_val->col_val_array[i].max_value ||
+             p_val->col_val_array[i].column_value.value_str);
     }
 #endif
     if (part_elem->list_val_list.push_back(p_val, mem_root)) return true;
@@ -1730,7 +1716,7 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
                                     partition_element *part_elem,
                                     const dd::Partition *part_obj,
                                     bool is_subpart) {
-  dd::String_type comment = part_obj->comment();
+  const dd::String_type comment = part_obj->comment();
   if (comment.length()) {
     part_elem->part_comment = strdup_root(mem_root, comment.c_str());
     if (!part_elem->part_comment) return true;
@@ -1757,10 +1743,10 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
       if (get_part_column_values(mem_root, part_info, part_elem, part_obj))
         return true;
     } else {
-      DBUG_ASSERT(part_obj->values().size() == 1);
+      assert(part_obj->values().size() == 1);
       const dd::Partition_value *part_value = *part_obj->values().begin();
-      DBUG_ASSERT(part_value->list_num() == 0);
-      DBUG_ASSERT(part_value->column_num() == 0);
+      assert(part_value->list_num() == 0);
+      assert(part_value->column_num() == 0);
       if (part_value->max_value()) {
         part_elem->max_value = true;
       } else {
@@ -1790,13 +1776,13 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
         max_index = std::max(max_index, part_value->list_num());
         entries++;
         if (part_value->value_utf8().empty()) {
-          DBUG_ASSERT(!part_elem->has_null_value);
+          assert(!part_elem->has_null_value);
           part_elem->has_null_value = true;
           null_entry = part_value->list_num();
         }
       }
       if (entries != (max_index + 1)) {
-        DBUG_ASSERT(0); /* purecov: deadcode */
+        assert(0); /* purecov: deadcode */
         return true;
       }
       /* If a list entry is NULL then it is only flagged on the part_elem. */
@@ -1810,9 +1796,9 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
       }
 
       for (const dd::Partition_value *part_value : part_obj->values()) {
-        DBUG_ASSERT(part_value->column_num() == 0);
+        assert(part_value->column_num() == 0);
         if (part_value->value_utf8().empty()) {
-          DBUG_ASSERT(part_value->list_num() == null_entry);
+          assert(part_value->list_num() == null_entry);
           continue;
         }
         list_index = part_value->list_num();
@@ -1824,7 +1810,7 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
         */
         if (part_elem->has_null_value && list_index > null_entry) list_index--;
         list_val = &list_val_array[list_index];
-        DBUG_ASSERT(!list_val->unsigned_flag && !list_val->value);
+        assert(!list_val->unsigned_flag && !list_val->value);
         if (part_value->value_utf8()[0] == '-') {
           list_val->unsigned_flag = false;
           if (dd::Properties::from_str(part_value->value_utf8(),
@@ -1843,9 +1829,9 @@ static bool setup_partition_from_dd(THD *thd, MEM_ROOT *mem_root,
       }
     }
   } else {
-#ifndef DBUG_OFF
-    DBUG_ASSERT(part_info->part_type == partition_type::HASH);
-    DBUG_ASSERT(part_obj->values().empty());
+#ifndef NDEBUG
+    assert(part_info->part_type == partition_type::HASH);
+    assert(part_obj->values().empty());
 #endif
   }
   return false;
@@ -1876,7 +1862,7 @@ static bool set_field_list(MEM_ROOT *mem_root, dd::String_type &str,
     if (dd::eat_str(field_name, it, end, dd::FIELD_NAME_SEPARATOR_CHAR))
       return true;
     size_t len = field_name.length();
-    DBUG_ASSERT(len);
+    assert(len);
     char *name = static_cast<char *>(mem_root->Alloc(len + 1));
     if (!name) return true; /* purecov: inspected */
     memcpy(name, field_name.c_str(), len);
@@ -1929,7 +1915,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
   handlerton *hton = plugin_data<handlerton *>(
       ha_resolve_by_name_raw(thd, lex_cstring_handle(tab_obj->engine())));
-  DBUG_ASSERT(hton && ha_storage_engine_is_enabled(hton));
+  assert(hton && ha_storage_engine_is_enabled(hton));
   part_info->default_engine_type = hton;
   if (!part_info->default_engine_type) return true;
 
@@ -1938,26 +1924,26 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
     case dd::Table::PT_RANGE_COLUMNS:
       part_info->column_list = true;
       part_info->list_of_part_fields = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::PT_RANGE:
       part_info->part_type = partition_type::RANGE;
       break;
     case dd::Table::PT_LIST_COLUMNS:
       part_info->column_list = true;
       part_info->list_of_part_fields = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::PT_LIST:
       part_info->part_type = partition_type::LIST;
       break;
     case dd::Table::PT_LINEAR_HASH:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::PT_HASH:
       part_info->part_type = partition_type::HASH;
       break;
     case dd::Table::PT_LINEAR_KEY_51:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::PT_KEY_51:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_51;
       part_info->list_of_part_fields = true;
@@ -1965,7 +1951,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::PT_LINEAR_KEY_55:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::PT_KEY_55:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_55;
       part_info->list_of_part_fields = true;
@@ -1973,7 +1959,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::PT_AUTO_LINEAR:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::PT_AUTO:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_55;
       part_info->part_type = partition_type::HASH;
@@ -1983,7 +1969,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     default:
       // Unknown partitioning type!
-      DBUG_ASSERT(0); /* purecov: deadcode */
+      assert(0); /* purecov: deadcode */
       return true;
   }
   switch (tab_obj->subpartition_type()) {
@@ -1992,13 +1978,13 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::ST_LINEAR_HASH:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::ST_HASH:
       part_info->subpart_type = partition_type::HASH;
       break;
     case dd::Table::ST_LINEAR_KEY_51:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::ST_KEY_51:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_51;
       part_info->list_of_subpart_fields = true;
@@ -2006,7 +1992,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::ST_LINEAR_KEY_55:
       part_info->linear_hash_ind = true;
-      // Fall through.
+      [[fallthrough]];
     case dd::Table::ST_KEY_55:
       part_info->key_algorithm = enum_key_algorithm::KEY_ALGORITHM_55;
       part_info->list_of_subpart_fields = true;
@@ -2014,7 +2000,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     default:
       // Unknown sub partitioning type!
-      DBUG_ASSERT(0); /* purecov: deadcode */
+      assert(0); /* purecov: deadcode */
       return true;
   }
 
@@ -2056,17 +2042,17 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
   /* Partitions are sorted first on level and then on number. */
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   uint number = 0;
 #endif
   for (const dd::Partition *part_obj : tab_obj->partitions()) {
-#ifndef DBUG_OFF
+#ifndef NDEBUG
     /* Must be in sorted order (sorted by level first and then on number). */
-    DBUG_ASSERT(part_obj->number() >= number);
+    assert(part_obj->number() >= number);
     number = part_obj->number();
 #endif
 
-    DBUG_ASSERT(part_obj->parent_partition_id() == dd::INVALID_OBJECT_ID);
+    assert(part_obj->parent_partition_id() == dd::INVALID_OBJECT_ID);
 
     curr_part_elem = new (&share->mem_root) partition_element;
     if (!curr_part_elem) {
@@ -2118,7 +2104,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       break;
     case dd::Table::DP_NONE:
     default:
-      DBUG_ASSERT(0); /* purecov: deadcode */
+      assert(0); /* purecov: deadcode */
   }
   switch (tab_obj->default_subpartitioning()) {
     case dd::Table::DP_NO:
@@ -2134,10 +2120,10 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
       part_info->use_default_num_subpartitions = false;
       break;
     case dd::Table::DP_NONE:
-      DBUG_ASSERT(!part_info->is_sub_partitioned());
+      assert(!part_info->is_sub_partitioned());
       break;
     default:
-      DBUG_ASSERT(0); /* purecov: deadcode */
+      assert(0); /* purecov: deadcode */
   }
 
   char *buf;
@@ -2145,7 +2131,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
   // Turn off ANSI_QUOTES and other SQL modes which affect printing of
   // generated partitioning clause.
-  Sql_mode_parse_guard parse_guard(thd);
+  const Sql_mode_parse_guard parse_guard(thd);
 
   buf = generate_partition_syntax(part_info, &buf_len, true, true, false,
                                   nullptr);
@@ -2166,7 +2152,7 @@ static bool fill_partitioning_from_dd(THD *thd, TABLE_SHARE *share,
 
 static bool fill_foreign_keys_from_dd(TABLE_SHARE *share,
                                       const dd::Table *tab_obj) {
-  DBUG_ASSERT(share->foreign_keys == 0 && share->foreign_key_parents == 0);
+  assert(share->foreign_keys == 0 && share->foreign_key_parents == 0);
 
   share->foreign_keys = tab_obj->foreign_keys().size();
   share->foreign_key_parents = tab_obj->foreign_key_parents().size();
@@ -2261,7 +2247,7 @@ static bool fill_foreign_keys_from_dd(TABLE_SHARE *share,
 */
 static bool fill_check_constraints_from_dd(TABLE_SHARE *share,
                                            const dd::Table *tab_obj) {
-  DBUG_ASSERT(share->check_constraint_share_list == nullptr);
+  assert(share->check_constraint_share_list == nullptr);
 
   if (tab_obj->check_constraints().size() > 0) {
     share->check_constraint_share_list = new (&share->mem_root)
@@ -2343,7 +2329,7 @@ bool open_table_def_suppress_invalid_meta_data(THD *thd, TABLE_SHARE *share,
                                                const dd::Table &table_def) {
   Open_table_error_handler error_handler;
   thd->push_internal_handler(&error_handler);
-  bool error = open_table_def(thd, share, table_def);
+  const bool error = open_table_def(thd, share, table_def);
   thd->pop_internal_handler();
   return error;
 }
