@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,13 +30,13 @@
 #include <sys/types.h>
 #include <time.h>
 #include <atomic>
+#include <string>
+#include <vector>
 
 #include <mysql/components/minimal_chassis.h>
 #include <mysql/components/services/dynamic_loader_scheme_file.h>
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "my_command.h"
-#include "my_compiler.h"
 #include "my_compress.h"
 #include "my_getopt.h"
 #include "my_hostname.h"  // HOSTNAME_LENGTH
@@ -47,18 +47,19 @@
 #include "my_sqlcommand.h"  // SQLCOM_END
 #include "my_sys.h"         // MY_TMPDIR
 #include "my_thread.h"      // my_thread_attr_t
-#include "mysql/components/services/mysql_cond_bits.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/mysql_rwlock_bits.h"
-#include "mysql/components/services/psi_cond_bits.h"
-#include "mysql/components/services/psi_file_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
-#include "mysql/components/services/psi_rwlock_bits.h"
-#include "mysql/components/services/psi_socket_bits.h"
-#include "mysql/components/services/psi_stage_bits.h"
-#include "mysql/components/services/psi_statement_bits.h"
-#include "mysql/components/services/psi_thread_bits.h"
+#include "mysql/components/services/bits/mysql_cond_bits.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/mysql_rwlock_bits.h"
+#include "mysql/components/services/bits/psi_cond_bits.h"
+#include "mysql/components/services/bits/psi_file_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/components/services/bits/psi_rwlock_bits.h"
+#include "mysql/components/services/bits/psi_socket_bits.h"
+#include "mysql/components/services/bits/psi_stage_bits.h"
+#include "mysql/components/services/bits/psi_statement_bits.h"
+#include "mysql/components/services/bits/psi_thread_bits.h"
 #include "mysql/status_var.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"  // SERVER_VERSION_LENGTH
 #ifdef _WIN32
 #include "sql/nt_servc.h"
@@ -67,6 +68,8 @@
 #include "sql/sql_const.h"  // UUID_LENGTH
 
 class Rpl_global_filter;
+class Rpl_acf_configuration_handler;
+class Source_IO_monitor;
 class THD;
 class Time_zone;
 struct MEM_ROOT;
@@ -102,6 +105,8 @@ typedef Bitmap<((MAX_INDEXES + 7) / 8 * 8)> Key_map; /* Used for finding keys */
 #define TEST_DO_QUICK_LEAK_CHECK       \
   4096 /**< Do Valgrind leak check for \
           each command. */
+#define TEST_NO_TEMP_TABLES \
+  8192 /**< No temp table engine is loaded, so use dummy costs. */
 
 #define SPECIAL_NO_NEW_FUNC 2     /* Skip new functions */
 #define SPECIAL_SKIP_SHOW_DB 4    /* Don't allow 'show db' */
@@ -123,11 +128,20 @@ extern bool dynamic_plugins_are_initialized;
 bool signal_restart_server();
 void kill_mysql(void);
 void refresh_status();
+void reset_status_by_thd();
 bool is_secure_file_path(const char *path);
 ulong sql_rnd_with_mutex();
 
 struct System_status_var *get_thd_status_var(THD *thd, bool *aggregated);
 
+#ifndef NDEBUG
+void thd_mem_cnt_alloc(THD *thd, size_t size, const char *key_name);
+#else
+void thd_mem_cnt_alloc(THD *thd, size_t size);
+#endif
+
+void thd_mem_cnt_free(THD *thd, size_t size);
+
 // These are needed for unit testing.
 void set_remaining_args(int argc, char **argv);
 int init_common_variables();
@@ -136,21 +150,16 @@ bool gtid_server_init();
 void gtid_server_cleanup();
 void clean_up_mysqld_mutexes();
 
-extern MYSQL_PLUGIN_IMPORT CHARSET_INFO *files_charset_info;
-extern MYSQL_PLUGIN_IMPORT CHARSET_INFO *national_charset_info;
-extern MYSQL_PLUGIN_IMPORT CHARSET_INFO *table_alias_charset;
-extern CHARSET_INFO *character_set_filesystem;
-
 enum enum_server_operational_state {
   SERVER_BOOTING,      /* Server is not operational. It is starting */
   SERVER_OPERATING,    /* Server is fully initialized and operating */
-  SERVER_SHUTTING_DOWN /* erver is shutting down */
+  SERVER_SHUTTING_DOWN /* Server is shutting down */
 };
 enum_server_operational_state get_server_state();
 
 extern bool opt_large_files, server_id_supplied;
 extern bool opt_bin_log;
-extern bool opt_log_slave_updates;
+extern bool opt_log_replica_updates;
 extern bool opt_log_unsafe_statements;
 extern bool opt_general_log, opt_slow_log, opt_general_log_raw;
 extern ulonglong log_output_options;
@@ -169,9 +178,11 @@ extern long opt_upgrade_mode;
 extern bool opt_initialize;
 extern bool opt_safe_user_create;
 extern bool opt_local_infile, opt_myisam_use_mmap;
-extern bool opt_slave_compressed_protocol;
-extern ulong slave_exec_mode_options;
+extern bool opt_replica_compressed_protocol;
+extern ulong replica_exec_mode_options;
 extern Rpl_global_filter rpl_global_filter;
+extern Rpl_acf_configuration_handler *rpl_acf_configuration_handler;
+extern Source_IO_monitor *rpl_source_io_monitor;
 extern int32_t opt_regexp_time_limit;
 extern int32_t opt_regexp_stack_limit;
 #ifdef _WIN32
@@ -180,13 +191,13 @@ extern bool opt_no_monitor;
 extern bool opt_debugging;
 extern bool opt_validate_config;
 
-enum enum_slave_type_conversions {
-  SLAVE_TYPE_CONVERSIONS_ALL_LOSSY,
-  SLAVE_TYPE_CONVERSIONS_ALL_NON_LOSSY,
-  SLAVE_TYPE_CONVERSIONS_ALL_UNSIGNED,
-  SLAVE_TYPE_CONVERSIONS_ALL_SIGNED
+enum enum_replica_type_conversions {
+  REPLICA_TYPE_CONVERSIONS_ALL_LOSSY,
+  REPLICA_TYPE_CONVERSIONS_ALL_NON_LOSSY,
+  REPLICA_TYPE_CONVERSIONS_ALL_UNSIGNED,
+  REPLICA_TYPE_CONVERSIONS_ALL_SIGNED
 };
-extern ulonglong slave_type_conversions_options;
+extern ulonglong replica_type_conversions_options;
 
 extern bool read_only, opt_readonly;
 extern bool super_read_only, opt_super_readonly;
@@ -200,10 +211,10 @@ enum enum_slave_rows_search_algorithms {
 extern ulonglong slave_rows_search_algorithms_options;
 extern bool opt_require_secure_transport;
 
-extern bool opt_slave_preserve_commit_order;
+extern bool opt_replica_preserve_commit_order;
 
-#ifndef DBUG_OFF
-extern uint slave_rows_last_search_algorithm_used;
+#ifndef NDEBUG
+extern uint replica_rows_last_search_algorithm_used;
 #endif
 extern ulong mts_parallel_option;
 #ifdef _WIN32
@@ -214,7 +225,7 @@ extern mysql_rwlock_t LOCK_named_pipe_full_access_group;
 #endif
 extern bool opt_allow_suspicious_udfs;
 extern const char *opt_secure_file_priv;
-extern bool opt_log_slow_admin_statements, opt_log_slow_slave_statements;
+extern bool opt_log_slow_admin_statements, opt_log_slow_replica_statements;
 extern bool sp_automatic_privileges, opt_noacl;
 extern bool opt_old_style_user_limits, trust_function_creators;
 extern bool check_proxy_users, mysql_native_password_proxy_users,
@@ -236,8 +247,9 @@ extern bool opt_using_transactions;
 extern ulong current_pid;
 extern ulong expire_logs_days;
 extern ulong binlog_expire_logs_seconds;
+extern bool opt_binlog_expire_logs_auto_purge;
 extern uint sync_binlog_period, sync_relaylog_period, sync_relayloginfo_period,
-    sync_masterinfo_period, opt_mts_checkpoint_period, opt_mts_checkpoint_group;
+    sync_masterinfo_period, opt_mta_checkpoint_period, opt_mta_checkpoint_group;
 extern ulong opt_tc_log_size, tc_log_max_pages_used, tc_log_page_size;
 extern ulong tc_log_page_waits;
 extern bool relay_log_purge;
@@ -279,17 +291,22 @@ extern char *my_admin_bind_addr_str;
 extern uint mysqld_admin_port;
 extern bool listen_admin_interface_in_separate_thread;
 extern char glob_hostname[HOSTNAME_LENGTH + 1];
-extern char system_time_zone[30], *opt_init_file;
+extern char system_time_zone_dst_on[30], system_time_zone_dst_off[30];
+extern char *opt_init_file;
 extern const char *opt_tc_log_file;
 extern char server_uuid[UUID_LENGTH + 1];
 extern const char *server_uuid_ptr;
+#if defined(HAVE_BUILD_ID_SUPPORT)
+extern char server_build_id[42];
+extern const char *server_build_id_ptr;
+#endif
 extern const double log_10[309];
 extern ulong binlog_cache_use, binlog_cache_disk_use;
 extern ulong binlog_stmt_cache_use, binlog_stmt_cache_disk_use;
 extern ulong aborted_threads;
 extern ulong delayed_insert_timeout;
 extern ulong delayed_insert_limit, delayed_queue_size;
-extern std::atomic<int32> atomic_slave_open_temp_tables;
+extern std::atomic<int32> atomic_replica_open_temp_tables;
 extern ulong slow_launch_time;
 extern ulong table_cache_size;
 extern ulong schema_def_size;
@@ -299,12 +316,12 @@ extern ulong tablespace_def_size;
 extern MYSQL_PLUGIN_IMPORT ulong max_connections;
 extern ulong max_digest_length;
 extern ulong max_connect_errors, connect_timeout;
-extern bool opt_slave_allow_batching;
+extern bool opt_replica_allow_batching;
 extern ulong slave_trans_retries;
-extern uint slave_net_timeout;
-extern ulong opt_mts_slave_parallel_workers;
+extern uint replica_net_timeout;
+extern ulong opt_mts_replica_parallel_workers;
 extern ulonglong opt_mts_pending_jobs_size_max;
-extern ulong rpl_stop_slave_timeout;
+extern ulong rpl_stop_replica_timeout;
 extern bool log_bin_use_v1_row_events;
 extern ulong what_to_log, flush_time;
 extern ulong max_prepared_stmt_count, prepared_stmt_count;
@@ -317,13 +334,13 @@ extern int32 opt_binlog_max_flush_queue_time;
 extern long opt_binlog_group_commit_sync_delay;
 extern ulong opt_binlog_group_commit_sync_no_delay_count;
 extern ulong max_binlog_size, max_relay_log_size;
-extern ulong slave_max_allowed_packet;
+extern ulong replica_max_allowed_packet;
 extern ulong binlog_row_event_max_size;
 extern ulong binlog_checksum_options;
 extern ulong binlog_row_metadata;
 extern const char *binlog_checksum_type_names[];
-extern bool opt_master_verify_checksum;
-extern bool opt_slave_sql_verify_checksum;
+extern bool opt_source_verify_checksum;
+extern bool opt_replica_sql_verify_checksum;
 extern uint32 gtid_executed_compression_period;
 extern bool binlog_gtid_simple_recovery;
 extern ulong binlog_error_action;
@@ -335,6 +352,8 @@ enum enum_binlog_error_action {
   ABORT_SERVER = 1
 };
 extern const char *binlog_error_action_list[];
+extern char *opt_authentication_policy;
+extern std::vector<std::string> authentication_policy_list;
 
 extern ulong stored_program_cache_size;
 extern ulong back_log;
@@ -365,7 +384,7 @@ extern "C" MYSQL_PLUGIN_IMPORT char **orig_argv;
 extern my_thread_attr_t connection_attrib;
 extern bool old_mode;
 extern bool avoid_temporal_upgrade;
-extern LEX_STRING opt_init_connect, opt_init_slave;
+extern LEX_STRING opt_init_connect, opt_init_replica;
 extern ulong connection_errors_internal;
 extern ulong connection_errors_peer_addr;
 extern char *opt_log_error_suppression_list;
@@ -385,6 +404,9 @@ extern char *opt_keyring_migration_socket;
 extern char *opt_keyring_migration_source;
 extern char *opt_keyring_migration_destination;
 extern ulong opt_keyring_migration_port;
+
+extern ulonglong global_conn_mem_limit;
+extern ulonglong global_conn_mem_counter;
 /**
   Variable to check if connection related options are set
   as part of keyring migration.
@@ -406,13 +428,14 @@ extern PSI_mutex_key key_LOCK_error_log;
 extern PSI_mutex_key key_LOCK_thd_data;
 extern PSI_mutex_key key_LOCK_thd_sysvar;
 extern PSI_mutex_key key_LOCK_thd_protocol;
+extern PSI_mutex_key key_LOCK_thd_security_ctx;
 extern PSI_mutex_key key_LOG_LOCK_log;
-extern PSI_mutex_key key_master_info_data_lock;
-extern PSI_mutex_key key_master_info_run_lock;
-extern PSI_mutex_key key_master_info_sleep_lock;
-extern PSI_mutex_key key_master_info_thd_lock;
-extern PSI_mutex_key key_master_info_rotate_lock;
-extern PSI_mutex_key key_mutex_slave_reporting_capability_err_lock;
+extern PSI_mutex_key key_source_info_data_lock;
+extern PSI_mutex_key key_source_info_run_lock;
+extern PSI_mutex_key key_source_info_sleep_lock;
+extern PSI_mutex_key key_source_info_thd_lock;
+extern PSI_mutex_key key_source_info_rotate_lock;
+extern PSI_mutex_key key_mutex_replica_reporting_capability_err_lock;
 extern PSI_mutex_key key_relay_log_info_data_lock;
 extern PSI_mutex_key key_relay_log_info_sleep_lock;
 extern PSI_mutex_key key_relay_log_info_thd_lock;
@@ -428,23 +451,21 @@ extern PSI_mutex_key key_LOCK_thd_query;
 extern PSI_mutex_key key_LOCK_cost_const;
 extern PSI_mutex_key key_LOCK_current_cond;
 extern PSI_mutex_key key_RELAYLOG_LOCK_commit;
-extern PSI_mutex_key key_RELAYLOG_LOCK_commit_queue;
-extern PSI_mutex_key key_RELAYLOG_LOCK_done;
-extern PSI_mutex_key key_RELAYLOG_LOCK_flush_queue;
 extern PSI_mutex_key key_RELAYLOG_LOCK_index;
 extern PSI_mutex_key key_RELAYLOG_LOCK_log;
 extern PSI_mutex_key key_RELAYLOG_LOCK_log_end_pos;
 extern PSI_mutex_key key_RELAYLOG_LOCK_sync;
-extern PSI_mutex_key key_RELAYLOG_LOCK_sync_queue;
 extern PSI_mutex_key key_RELAYLOG_LOCK_xids;
 extern PSI_mutex_key key_gtid_ensure_index_mutex;
-extern PSI_mutex_key key_mts_temp_table_LOCK;
-extern PSI_mutex_key key_mts_gaq_LOCK;
+extern PSI_mutex_key key_mta_temp_table_LOCK;
+extern PSI_mutex_key key_mta_gaq_LOCK;
 extern PSI_mutex_key key_thd_timer_mutex;
 extern PSI_mutex_key key_monitor_info_run_lock;
+extern PSI_mutex_key key_LOCK_delegate_connection_mutex;
+extern PSI_mutex_key key_LOCK_group_replication_connection_mutex;
 
 extern PSI_mutex_key key_commit_order_manager_mutex;
-extern PSI_mutex_key key_mutex_slave_worker_hash;
+extern PSI_mutex_key key_mutex_replica_worker_hash;
 
 extern PSI_rwlock_key key_rwlock_LOCK_logger;
 extern PSI_rwlock_key key_rwlock_channel_map_lock;
@@ -460,11 +481,11 @@ extern PSI_cond_key key_COND_active;
 extern PSI_cond_key key_COND_pool;
 extern PSI_cond_key key_COND_cache_status_changed;
 extern PSI_cond_key key_item_func_sleep_cond;
-extern PSI_cond_key key_master_info_data_cond;
-extern PSI_cond_key key_master_info_start_cond;
-extern PSI_cond_key key_master_info_stop_cond;
-extern PSI_cond_key key_master_info_sleep_cond;
-extern PSI_cond_key key_master_info_rotate_cond;
+extern PSI_cond_key key_source_info_data_cond;
+extern PSI_cond_key key_source_info_start_cond;
+extern PSI_cond_key key_source_info_stop_cond;
+extern PSI_cond_key key_source_info_sleep_cond;
+extern PSI_cond_key key_source_info_rotate_cond;
 extern PSI_cond_key key_relay_log_info_data_cond;
 extern PSI_cond_key key_relay_log_info_log_space_cond;
 extern PSI_cond_key key_relay_log_info_start_cond;
@@ -472,14 +493,13 @@ extern PSI_cond_key key_relay_log_info_stop_cond;
 extern PSI_cond_key key_relay_log_info_sleep_cond;
 extern PSI_cond_key key_cond_slave_parallel_pend_jobs;
 extern PSI_cond_key key_cond_slave_parallel_worker;
-extern PSI_cond_key key_cond_mts_gaq;
-extern PSI_cond_key key_RELAYLOG_COND_done;
+extern PSI_cond_key key_cond_mta_gaq;
 extern PSI_cond_key key_RELAYLOG_update_cond;
-extern PSI_cond_key key_RELAYLOG_prep_xids_cond;
 extern PSI_cond_key key_gtid_ensure_index_cond;
 extern PSI_cond_key key_COND_thr_lock;
 extern PSI_cond_key key_cond_slave_worker_hash;
 extern PSI_cond_key key_commit_order_manager_cond;
+extern PSI_cond_key key_COND_group_replication_connection_cond_var;
 extern PSI_thread_key key_thread_bootstrap;
 extern PSI_thread_key key_thread_handle_manager;
 extern PSI_thread_key key_thread_one_connection;
@@ -527,13 +547,13 @@ extern PSI_stage_info stage_after_create;
 extern PSI_stage_info stage_alter_inplace_prepare;
 extern PSI_stage_info stage_alter_inplace;
 extern PSI_stage_info stage_alter_inplace_commit;
-extern PSI_stage_info stage_changing_master;
-extern PSI_stage_info stage_checking_master_version;
+extern PSI_stage_info stage_changing_source;
+extern PSI_stage_info stage_checking_source_version;
 extern PSI_stage_info stage_checking_permissions;
 extern PSI_stage_info stage_cleaning_up;
 extern PSI_stage_info stage_closing_tables;
 extern PSI_stage_info stage_compressing_gtid_table;
-extern PSI_stage_info stage_connecting_to_master;
+extern PSI_stage_info stage_connecting_to_source;
 extern PSI_stage_info stage_converting_heap_to_ondisk;
 extern PSI_stage_info stage_copy_to_tmp_table;
 extern PSI_stage_info stage_creating_table;
@@ -547,38 +567,49 @@ extern PSI_stage_info stage_execution_of_init_command;
 extern PSI_stage_info stage_explaining;
 extern PSI_stage_info
     stage_finished_reading_one_binlog_switching_to_next_binlog;
-extern PSI_stage_info stage_flushing_relay_log_and_master_info_repository;
+extern PSI_stage_info stage_flushing_relay_log_and_source_info_repository;
 extern PSI_stage_info stage_flushing_relay_log_info_file;
 extern PSI_stage_info stage_freeing_items;
 extern PSI_stage_info stage_fulltext_initialization;
 extern PSI_stage_info stage_init;
-extern PSI_stage_info stage_killing_slave;
+extern PSI_stage_info stage_killing_replica;
 extern PSI_stage_info stage_logging_slow_query;
 extern PSI_stage_info stage_making_temp_file_append_before_load_data;
 extern PSI_stage_info stage_manage_keys;
-extern PSI_stage_info stage_master_has_sent_all_binlog_to_slave;
+extern PSI_stage_info stage_source_has_sent_all_binlog_to_replica;
 extern PSI_stage_info stage_opening_tables;
 extern PSI_stage_info stage_optimizing;
 extern PSI_stage_info stage_preparing;
 extern PSI_stage_info stage_purging_old_relay_logs;
 extern PSI_stage_info stage_query_end;
-extern PSI_stage_info stage_queueing_master_event_to_the_relay_log;
+extern PSI_stage_info stage_queueing_source_event_to_the_relay_log;
 extern PSI_stage_info stage_reading_event_from_the_relay_log;
-extern PSI_stage_info stage_registering_slave_on_master;
+extern PSI_stage_info stage_registering_replica_on_source;
 extern PSI_stage_info stage_removing_tmp_table;
 extern PSI_stage_info stage_rename;
 extern PSI_stage_info stage_rename_result_table;
 extern PSI_stage_info stage_requesting_binlog_dump;
 extern PSI_stage_info stage_searching_rows_for_update;
-extern PSI_stage_info stage_sending_binlog_event_to_slave;
+extern PSI_stage_info stage_sending_binlog_event_to_replica;
 extern PSI_stage_info stage_setup;
-extern PSI_stage_info stage_slave_has_read_all_relay_log;
-extern PSI_stage_info stage_slave_waiting_event_from_coordinator;
-extern PSI_stage_info stage_slave_waiting_for_workers_to_process_queue;
-extern PSI_stage_info stage_slave_waiting_worker_queue;
-extern PSI_stage_info stage_slave_waiting_worker_to_free_events;
-extern PSI_stage_info stage_slave_waiting_worker_to_release_partition;
-extern PSI_stage_info stage_slave_waiting_workers_to_exit;
+extern PSI_stage_info stage_replica_has_read_all_relay_log;
+extern PSI_stage_info
+    stage_replica_reconnecting_after_failed_binlog_dump_request;
+extern PSI_stage_info stage_replica_reconnecting_after_failed_event_read;
+extern PSI_stage_info
+    stage_replica_reconnecting_after_failed_registration_on_source;
+extern PSI_stage_info stage_replica_waiting_event_from_coordinator;
+extern PSI_stage_info stage_replica_waiting_for_workers_to_process_queue;
+extern PSI_stage_info
+    stage_replica_waiting_to_reconnect_after_failed_binlog_dump_request;
+extern PSI_stage_info
+    stage_replica_waiting_to_reconnect_after_failed_event_read;
+extern PSI_stage_info
+    stage_replica_waiting_to_reconnect_after_failed_registration_on_source;
+extern PSI_stage_info stage_replica_waiting_worker_queue;
+extern PSI_stage_info stage_replica_waiting_worker_to_free_events;
+extern PSI_stage_info stage_replica_waiting_worker_to_release_partition;
+extern PSI_stage_info stage_replica_waiting_workers_to_exit;
 extern PSI_stage_info stage_rpl_apply_row_evt_write;
 extern PSI_stage_info stage_rpl_apply_row_evt_update;
 extern PSI_stage_info stage_rpl_apply_row_evt_delete;
@@ -593,14 +624,14 @@ extern PSI_stage_info stage_user_sleep;
 extern PSI_stage_info stage_verifying_table;
 extern PSI_stage_info stage_waiting_for_gtid_to_be_committed;
 extern PSI_stage_info stage_waiting_for_handler_commit;
-extern PSI_stage_info stage_waiting_for_master_to_send_event;
-extern PSI_stage_info stage_waiting_for_master_update;
+extern PSI_stage_info stage_waiting_for_source_to_send_event;
+extern PSI_stage_info stage_waiting_for_source_update;
 extern PSI_stage_info stage_waiting_for_relay_log_space;
-extern PSI_stage_info stage_waiting_for_slave_mutex_on_exit;
-extern PSI_stage_info stage_waiting_for_slave_thread_to_start;
+extern PSI_stage_info stage_waiting_for_replica_mutex_on_exit;
+extern PSI_stage_info stage_waiting_for_replica_thread_to_start;
 extern PSI_stage_info stage_waiting_for_table_flush;
 extern PSI_stage_info stage_waiting_for_the_next_event_in_relay_log;
-extern PSI_stage_info stage_waiting_for_the_slave_thread_to_advance_position;
+extern PSI_stage_info stage_waiting_for_the_replica_thread_to_advance_position;
 extern PSI_stage_info stage_waiting_to_finalize_termination;
 extern PSI_stage_info stage_worker_waiting_for_its_turn_to_commit;
 extern PSI_stage_info stage_worker_waiting_for_commit_parent;
@@ -613,6 +644,8 @@ extern PSI_stage_info stage_binlog_transaction_decompress;
 extern PSI_stage_info stage_rpl_failover_fetching_source_member_details;
 extern PSI_stage_info stage_rpl_failover_updating_source_member_details;
 extern PSI_stage_info stage_rpl_failover_wait_before_next_fetch;
+extern PSI_stage_info stage_communication_delegation;
+extern PSI_stage_info stage_wait_on_commit_ticket;
 #ifdef HAVE_PSI_STATEMENT_INTERFACE
 /**
   Statement instrumentation keys (sql).
@@ -671,10 +704,11 @@ extern mysql_mutex_t LOCK_global_system_variables;
 extern mysql_mutex_t LOCK_user_conn;
 extern mysql_mutex_t LOCK_log_throttle_qni;
 extern mysql_mutex_t LOCK_prepared_stmt_count;
+extern mysql_mutex_t LOCK_replica_list;
 extern mysql_mutex_t LOCK_error_messages;
-extern mysql_mutex_t LOCK_sql_slave_skip_counter;
-extern mysql_mutex_t LOCK_slave_net_timeout;
-extern mysql_mutex_t LOCK_slave_trans_dep_tracker;
+extern mysql_mutex_t LOCK_sql_replica_skip_counter;
+extern mysql_mutex_t LOCK_replica_net_timeout;
+extern mysql_mutex_t LOCK_replica_trans_dep_tracker;
 extern mysql_mutex_t LOCK_mandatory_roles;
 extern mysql_mutex_t LOCK_password_history;
 extern mysql_mutex_t LOCK_password_reuse_interval;
@@ -688,13 +722,15 @@ extern mysql_mutex_t LOCK_tls_ctx_options;
 extern mysql_mutex_t LOCK_admin_tls_ctx_options;
 extern mysql_mutex_t LOCK_rotate_binlog_master_key;
 extern mysql_mutex_t LOCK_partial_revokes;
+extern mysql_mutex_t LOCK_authentication_policy;
+extern mysql_mutex_t LOCK_global_conn_mem_limit;
 
 extern mysql_cond_t COND_server_started;
 extern mysql_cond_t COND_compress_gtid_table;
 extern mysql_cond_t COND_manager;
 
 extern mysql_rwlock_t LOCK_sys_init_connect;
-extern mysql_rwlock_t LOCK_sys_init_slave;
+extern mysql_rwlock_t LOCK_sys_init_replica;
 extern mysql_rwlock_t LOCK_system_variables_hash;
 
 extern ulong opt_ssl_fips_mode;
@@ -710,15 +746,14 @@ int *get_remaining_argc();
 char ***get_remaining_argv();
 
 /* increment query_id and return it.  */
-inline MY_ATTRIBUTE((warn_unused_result)) query_id_t next_query_id() {
+[[nodiscard]] inline query_id_t next_query_id() {
   return ++atomic_global_query_id;
 }
 
 #define ER(X) please_use_ER_THD_or_ER_DEFAULT_instead(X)
 
 /* Accessor function for _connection_events_loop_aborted flag */
-inline MY_ATTRIBUTE(
-    (warn_unused_result)) bool connection_events_loop_aborted() {
+[[nodiscard]] inline bool connection_events_loop_aborted() {
   return connection_events_loop_aborted_flag.load();
 }
 
@@ -772,6 +807,10 @@ void set_mysqld_partial_revokes(bool value);
 
 bool check_and_update_partial_revokes_sysvar(THD *thd);
 
+bool parse_authentication_policy(char *val,
+                                 std::vector<std::string> &policy_list);
+bool validate_authentication_policy(char *val);
+bool update_authentication_policy();
 #ifdef _WIN32
 
 bool is_windows_service();
@@ -793,4 +832,11 @@ extern SERVICE_TYPE_NO_CONST(registry) * srv_registry;
    mysql_server component */
 extern SERVICE_TYPE(dynamic_loader_scheme_file) * scheme_file_srv;
 extern SERVICE_TYPE(dynamic_loader) * dynamic_loader_srv;
+
+class Deployed_components;
+extern Deployed_components *g_deployed_components;
+
+extern bool opt_persist_sensitive_variables_in_plaintext;
+
+void persisted_variables_refresh_keyring_support();
 #endif /* MYSQLD_INCLUDED */
