@@ -30,20 +30,28 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_HINT_PARSER_VAR_LIB_PB2_SB_1_11874992_1687454811_68_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
-# define YY_HINT_PARSER_VAR_LIB_PB2_SB_1_11874992_1687454811_68_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
+#ifndef YY_MY_HINT_PARSER_VAR_LIB_PB2_SB_1_11858368_1687374168_0_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
+# define YY_MY_HINT_PARSER_VAR_LIB_PB2_SB_1_11858368_1687374168_0_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
 /* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
+#ifndef MY_HINT_PARSER_DEBUG
+# if defined YYDEBUG
 #if YYDEBUG
-extern int HINT_PARSER_debug;
+#   define MY_HINT_PARSER_DEBUG 1
+#  else
+#   define MY_HINT_PARSER_DEBUG 0
+#  endif
+# else /* ! defined YYDEBUG */
+#  define MY_HINT_PARSER_DEBUG 0
+# endif /* ! defined YYDEBUG */
+#endif  /* ! defined MY_HINT_PARSER_DEBUG */
+#if MY_HINT_PARSER_DEBUG
+extern int my_hint_parser_debug;
 #endif
 
 /* Token type.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-  enum yytokentype
+#ifndef MY_HINT_PARSER_TOKENTYPE
+# define MY_HINT_PARSER_TOKENTYPE
+  enum my_hint_parser_tokentype
   {
     MAX_EXECUTION_TIME_HINT = 1000,
     RESOURCE_GROUP_HINT = 1001,
@@ -94,8 +102,7 @@ extern int HINT_PARSER_debug;
     NO_ORDER_INDEX_HINT = 1046,
     DERIVED_CONDITION_PUSHDOWN_HINT = 1047,
     NO_DERIVED_CONDITION_PUSHDOWN_HINT = 1048,
-    HINT_ARG_FLOATING_POINT_NUMBER = 1049,
-    YYUNDEF = 1150
+    HINT_ARG_FLOATING_POINT_NUMBER = 1049
   };
 #endif
 /* Tokens.  */
@@ -149,12 +156,11 @@ extern int HINT_PARSER_debug;
 #define DERIVED_CONDITION_PUSHDOWN_HINT 1047
 #define NO_DERIVED_CONDITION_PUSHDOWN_HINT 1048
 #define HINT_ARG_FLOATING_POINT_NUMBER 1049
-#define YYUNDEF 1150
 
 /* Value type.  */
 
 
 
-int HINT_PARSER_parse (class THD *thd, class Hint_scanner *scanner, class PT_hint_list **ret);
+int my_hint_parser_parse (class THD *thd, class Hint_scanner *scanner, class PT_hint_list **ret);
 
-#endif /* !YY_HINT_PARSER_VAR_LIB_PB2_SB_1_11874992_1687454811_68_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED  */
+#endif /* !YY_MY_HINT_PARSER_VAR_LIB_PB2_SB_1_11858368_1687374168_0_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED  */
