@@ -30,20 +30,28 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MYSQL_VAR_LIB_PB2_SB_1_11874992_1687454811_68_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MYSQL_VAR_LIB_PB2_SB_1_11874992_1687454811_68_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_11858368_1687374168_0_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_11858368_1687374168_0_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
+#ifndef MY_SQL_PARSER_DEBUG
+# if defined YYDEBUG
 #if YYDEBUG
-extern int MYSQLdebug;
+#   define MY_SQL_PARSER_DEBUG 1
+#  else
+#   define MY_SQL_PARSER_DEBUG 0
+#  endif
+# else /* ! defined YYDEBUG */
+#  define MY_SQL_PARSER_DEBUG 0
+# endif /* ! defined YYDEBUG */
+#endif  /* ! defined MY_SQL_PARSER_DEBUG */
+#if MY_SQL_PARSER_DEBUG
+extern int my_sql_parser_debug;
 #endif
 
 /* Token type.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-  enum yytokentype
+#ifndef MY_SQL_PARSER_TOKENTYPE
+# define MY_SQL_PARSER_TOKENTYPE
+  enum my_sql_parser_tokentype
   {
     ABORT_SYM = 258,
     ACCESSIBLE_SYM = 259,
@@ -840,11 +848,13 @@ extern int MYSQLdebug;
     BULK_SYM = 1201,
     URL_SYM = 1202,
     GENERATE_SYM = 1203,
-    KEYWORD_USED_AS_IDENT = 1205,
-    KEYWORD_USED_AS_KEYWORD = 1206,
-    CONDITIONLESS_JOIN = 1207,
-    SUBQUERY_AS_EXPR = 1208,
-    EMPTY_FROM_CLAUSE = 1209
+    DOLLAR_QUOTED_STRING_SYM = 1204,
+    PARSE_TREE_SYM = 1205,
+    KEYWORD_USED_AS_IDENT = 1207,
+    KEYWORD_USED_AS_KEYWORD = 1208,
+    CONDITIONLESS_JOIN = 1209,
+    SUBQUERY_AS_EXPR = 1210,
+    EMPTY_FROM_CLAUSE = 1211
   };
 #endif
 /* Tokens.  */
@@ -1643,30 +1653,32 @@ extern int MYSQLdebug;
 #define BULK_SYM 1201
 #define URL_SYM 1202
 #define GENERATE_SYM 1203
-#define KEYWORD_USED_AS_IDENT 1205
-#define KEYWORD_USED_AS_KEYWORD 1206
-#define CONDITIONLESS_JOIN 1207
-#define SUBQUERY_AS_EXPR 1208
-#define EMPTY_FROM_CLAUSE 1209
+#define DOLLAR_QUOTED_STRING_SYM 1204
+#define PARSE_TREE_SYM 1205
+#define KEYWORD_USED_AS_IDENT 1207
+#define KEYWORD_USED_AS_KEYWORD 1208
+#define CONDITIONLESS_JOIN 1209
+#define SUBQUERY_AS_EXPR 1210
+#define EMPTY_FROM_CLAUSE 1211
 
 /* Value type.  */
 
 /* Location type.  */
-#if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
-typedef struct YYLTYPE YYLTYPE;
-struct YYLTYPE
+#if ! defined MY_SQL_PARSER_LTYPE && ! defined MY_SQL_PARSER_LTYPE_IS_DECLARED
+typedef struct MY_SQL_PARSER_LTYPE MY_SQL_PARSER_LTYPE;
+struct MY_SQL_PARSER_LTYPE
 {
   int first_line;
   int first_column;
   int last_line;
   int last_column;
 };
-# define YYLTYPE_IS_DECLARED 1
-# define YYLTYPE_IS_TRIVIAL 1
+# define MY_SQL_PARSER_LTYPE_IS_DECLARED 1
+# define MY_SQL_PARSER_LTYPE_IS_TRIVIAL 1
 #endif
 
 
 
-int MYSQLparse (class THD *YYTHD, class Parse_tree_root **parse_tree);
+int my_sql_parser_parse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MYSQL_VAR_LIB_PB2_SB_1_11874992_1687454811_68_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_11858368_1687374168_0_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
