@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2020, Oracle and/or its affiliates.
+Copyright (c) 1997, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -36,10 +36,12 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "buf0types.h"
 #include "dict0types.h"
 #include "hash0hash.h"
-#include "log0types.h"
+#include "log0sys.h"
 #include "mtr0types.h"
-#include "os0file.h" /* OS_FILE_LOG_BLOCK_SIZE */
-#include "univ.i"
+
+/* OS_FILE_LOG_BLOCK_SIZE */
+#include "os0file.h"
+
 #include "ut0byte.h"
 #include "ut0new.h"
 
@@ -58,107 +60,90 @@ struct recv_addr_t;
 extern std::list<std::pair<space_id_t, lsn_t>> index_load_list;
 /** the last redo log flush len as seen by MEB */
 extern volatile lsn_t backup_redo_log_flushed_lsn;
-/** TRUE when the redo log is being backed up */
+/** true when the redo log is being backed up */
 extern bool recv_is_making_a_backup;
 
 /** Scans the log segment and n_bytes_scanned is set to the length of valid
 log scanned.
-@param[in]	buf			buffer containing log data
-@param[in]	buf_len			data length in that buffer
-@param[in,out]	scanned_lsn		lsn of buffer start, we return scanned
+@param[in]      buf                     buffer containing log data
+@param[in]      buf_len                 data length in that buffer
+@param[in,out]  scanned_lsn             lsn of buffer start, we return scanned
 lsn
-@param[in,out]	scanned_checkpoint_no	4 lowest bytes of the highest scanned
-@param[out]	block_no	highest block no in scanned buffer.
+@param[in,out]  scanned_checkpoint_no   4 lowest bytes of the highest scanned
+@param[out]     block_no        highest block no in scanned buffer.
 checkpoint number so far
-@param[out]	n_bytes_scanned		how much we were able to scan, smaller
+@param[out]     n_bytes_scanned         how much we were able to scan, smaller
 than buf_len if log data ended here
-@param[out]	has_encrypted_log	set true, if buffer contains encrypted
+@param[out]     has_encrypted_log       set true, if buffer contains encrypted
 redo log, set false otherwise */
-void meb_scan_log_seg(byte *buf, ulint buf_len, lsn_t *scanned_lsn,
+void meb_scan_log_seg(byte *buf, size_t buf_len, lsn_t *scanned_lsn,
                       uint32_t *scanned_checkpoint_no, uint32_t *block_no,
-                      ulint *n_bytes_scanned, bool *has_encrypted_log);
+                      size_t *n_bytes_scanned, bool *has_encrypted_log);
 
 /** Applies the hashed log records to the page, if the page lsn is less than the
 lsn of a log record. This can be called when a buffer page has just been
 read in, or also for a page already in the buffer pool.
 
 
-@param[in,out]	block		buffer block */
-/* TODO(Bug#31173032): Remove SUPPRESS_UBSAN_CLANG10. */
-void recv_recover_page_func(buf_block_t *block) SUPPRESS_UBSAN_CLANG10;
+@param[in,out]  block           buffer block */
+void recv_recover_page_func(buf_block_t *block);
 
 /** Wrapper for recv_recover_page_func().
 Applies the hashed log records to the page, if the page lsn is less than the
 lsn of a log record. This can be called when a buffer page has just been
 read in, or also for a page already in the buffer pool.
-@param jri in: TRUE if just read in (the i/o handler calls this for
+@param jri in: true if just read in (the i/o handler calls this for
 a freshly read page)
 @param block in,out: the buffer block
 */
-#define recv_recover_page(jri, block) recv_recover_page_func(block)
+static inline void recv_recover_page(bool jri [[maybe_unused]],
+                                     buf_block_t *block) {
+  recv_recover_page_func(block);
+}
 
 /** Applies log records in the hash table to a backup. */
 void meb_apply_log_recs(void);
 
 /** Applies log records in the hash table to a backup using a callback
 functions.
-@param[in]	apply_log_record_function  function for apply
-@param[in]	wait_till_done_function    function for wait */
+@param[in]      apply_log_record_function  function for apply
+@param[in]      wait_till_done_function    function for wait */
 void meb_apply_log_recs_via_callback(
     void (*apply_log_record_function)(recv_addr_t *),
     void (*wait_till_done_function)());
 
 /** Applies a log record in the hash table to a backup.
-@param[in]	recv_addr	chain of log records
-@param[in,out]	block		buffer block to apply the records to */
+@param[in]      recv_addr       chain of log records
+@param[in,out]  block           buffer block to apply the records to */
 void meb_apply_log_record(recv_addr_t *recv_addr, buf_block_t *block);
 
 /** Process a file name passed as an input
-@param[in]	name		absolute path of tablespace file
-@param[in]	space_id	the tablespace ID
-@retval		true		if able to process file successfully.
-@retval		false		if unable to process the file */
+@param[in]      name            absolute path of tablespace file
+@param[in]      space_id        the tablespace ID
+@retval         true            if able to process file successfully.
+@retval         false           if unable to process the file */
 void meb_fil_name_process(const char *name, space_id_t space_id);
 
 /** Scans log from a buffer and stores new log data to the parsing buffer.
 Parses and hashes the log records if new data found.  Unless
 UNIV_HOTBACKUP is defined, this function will apply log records
 automatically when the hash table becomes full.
-@param[in]	available_memory	we let the hash table of recs
+@param[in]      available_memory        we let the hash table of recs
 to grow to this size, at the maximum
-@param[in]	buf			buffer containing a log
+@param[in]      buf                     buffer containing a log
 segment or garbage
-@param[in]	len			buffer length
-@param[in]	checkpoint_lsn		latest checkpoint LSN
-@param[in]	start_lsn		buffer start lsn
-@param[in]	contiguous_lsn		it is known that all log
-groups contain contiguous log data up to this lsn
-@param[out]	group_scanned_lsn	scanning succeeded up to this lsn
-@retval	true	if limit_lsn has been reached, or not able to scan any
+@param[in]      len                     buffer length
+@param[in]      start_lsn               buffer start lsn
+@param[out]     group_scanned_lsn       scanning succeeded up to this lsn
+@retval	true  if limit_lsn has been reached, or not able to scan any
 more in this log group
-@retval	false	otherwise */
-bool meb_scan_log_recs(ulint available_memory, const byte *buf, ulint len,
-                       lsn_t checkpoint_lsn, lsn_t start_lsn,
-                       lsn_t *contiguous_lsn, lsn_t *group_scanned_lsn);
+@retval false   otherwise */
+bool meb_scan_log_recs(size_t available_memory, const byte *buf, size_t len,
+                       lsn_t start_lsn, lsn_t *group_scanned_lsn);
 
-/** Creates an IORequest object for decrypting redo log with
-Encryption::decrypt_log() method. If the encryption_info parameter is
-a null pointer, then encryption information is read from
-"ib_logfile0". If the encryption_info parameter is not null, then it
-should contain a copy of the encryption info stored in the header of
-"ib_logfile0".
-@param[in,out]	encryption_request      an IORequest object
-@param[in]	encryption_info         a copy of the encryption info in
-the header of "ib_logfile0", or a null pointer
-@retval	true	if the call succeeded
-@retval	false	otherwise */
-bool meb_read_log_encryption(IORequest &encryption_request,
-                             byte *encryption_info = nullptr);
-
-bool recv_check_log_header_checksum(const byte *buf);
 /** Check the 4-byte checksum to the trailer checksum field of a log
 block.
-@param[in]	block	pointer to a log block
+@param[in]      block   pointer to a log block
 @return whether the checksum matches */
 bool log_block_checksum_is_ok(const byte *block);
 #else /* UNIV_HOTBACKUP */
@@ -168,21 +153,21 @@ lsn of a log record. This can be called when a buffer page has just been
 read in, or also for a page already in the buffer pool.
 
 
-@param[in]	just_read_in	true if the IO handler calls this for a freshly
+@param[in]      just_read_in    true if the IO handler calls this for a freshly
                                 read page
-@param[in,out]	block		buffer block */
-/* TODO(fix Bug#31173032): Remove SUPPRESS_UBSAN_CLANG10. */
-void recv_recover_page_func(bool just_read_in,
-                            buf_block_t *block) SUPPRESS_UBSAN_CLANG10;
+@param[in,out]  block           buffer block */
+void recv_recover_page_func(bool just_read_in, buf_block_t *block);
 
 /** Wrapper for recv_recover_page_func().
 Applies the hashed log records to the page, if the page lsn is less than the
 lsn of a log record. This can be called when a buffer page has just been
 read in, or also for a page already in the buffer pool.
-@param jri in: TRUE if just read in (the i/o handler calls this for
+@param jri in: true if just read in (the i/o handler calls this for
 a freshly read page)
-@param[in,out]	block	buffer block */
-#define recv_recover_page(jri, block) recv_recover_page_func(jri, block)
+@param[in,out]  block   buffer block */
+static inline void recv_recover_page(bool jri, buf_block_t *block) {
+  recv_recover_page_func(jri, block);
+}
 
 #endif /* UNIV_HOTBACKUP */
 
@@ -200,24 +185,35 @@ size_t meb_heap_used();
 
 /** Returns true if recovery is currently running.
 @return recv_recovery_on */
-UNIV_INLINE
-bool recv_recovery_is_on() MY_ATTRIBUTE((warn_unused_result));
+[[nodiscard]] static inline bool recv_recovery_is_on();
+
+/** Returns true if the page is brand new (the next log record is init_file_page
+or no records to apply).
+@param[in]      block           buffer block
+@return true if brand new */
+bool recv_page_is_brand_new(buf_block_t *block);
 
 /** Start recovering from a redo log checkpoint.
 @see recv_recovery_from_checkpoint_finish
-@param[in,out]	log		redo log
-@param[in]	flush_lsn	FIL_PAGE_FILE_FLUSH_LSN
-                                of first system tablespace page
+@param[in,out]  log        redo log
+@param[in]      flush_lsn  lsn stored at offset FIL_PAGE_FILE_FLUSH_LSN
+                           in the system tablespace header
 @return error code or DB_SUCCESS */
-dberr_t recv_recovery_from_checkpoint_start(log_t &log, lsn_t flush_lsn)
-    MY_ATTRIBUTE((warn_unused_result));
+[[nodiscard]] dberr_t recv_recovery_from_checkpoint_start(log_t &log,
+                                                          lsn_t flush_lsn);
+
+/** Determine if a redo log from a version before MySQL 8.0.30 is clean.
+@param[in,out]  log             redo log
+@return error code
+@retval DB_SUCCESS  if the redo log is clean
+@retval DB_ERROR    if the redo log is corrupted or dirty */
+dberr_t recv_verify_log_is_clean_pre_8_0_30(log_t &log);
 
 /** Complete the recovery from the latest checkpoint.
-@param[in,out]	log		redo log
-@param[in]	aborting	true if the server has to abort due to an error
+@param[in]      aborting        true if the server has to abort due to an error
 @return recovered persistent metadata or nullptr if aborting*/
-MetadataRecover *recv_recovery_from_checkpoint_finish(log_t &log, bool aborting)
-    MY_ATTRIBUTE((warn_unused_result));
+[[nodiscard]] MetadataRecover *recv_recovery_from_checkpoint_finish(
+    bool aborting);
 
 /** Creates the recovery system. */
 void recv_sys_create();
@@ -225,21 +221,20 @@ void recv_sys_create();
 /** Release recovery system mutexes. */
 void recv_sys_close();
 
-/** Inits the recovery system for a recovery operation.
-@param[in]	max_mem		Available memory in bytes */
-void recv_sys_init(ulint max_mem);
+/** Inits the recovery system for a recovery operation. */
+void recv_sys_init();
 
 /** Calculates the new value for lsn when more data is added to the log.
-@param[in]	lsn		Old LSN
-@param[in]	len		This many bytes of data is added, log block
+@param[in]      lsn             Old LSN
+@param[in]      len             This many bytes of data is added, log block
                                 headers not included
 @return LSN after data addition */
-lsn_t recv_calc_lsn_on_data_add(lsn_t lsn, uint64_t len);
+lsn_t recv_calc_lsn_on_data_add(lsn_t lsn, os_offset_t len);
 
 /** Empties the hash table of stored log records, applying them to appropriate
 pages.
-@param[in,out]	log		Redo log
-@param[in]	allow_ibuf	if true, ibuf operations are allowed during
+@param[in,out]  log             redo log
+@param[in]      allow_ibuf      if true, ibuf operations are allowed during
                                 the application; if false, no ibuf operations
                                 are allowed, and after the application all
                                 file pages are flushed to disk and invalidated
@@ -247,11 +242,11 @@ pages.
                                 no new log records can be generated during
                                 the application; the caller must in this case
                                 own the log mutex */
-void recv_apply_hashed_log_recs(log_t &log, bool allow_ibuf);
+dberr_t recv_apply_hashed_log_recs(log_t &log, bool allow_ibuf);
 
 #if defined(UNIV_DEBUG) || defined(UNIV_HOTBACKUP)
 /** Return string name of the redo log record type.
-@param[in]	type	record log record enum
+@param[in]      type    record log record enum
 @return string name of record log record */
 const char *get_mlog_string(mlog_id_t type);
 #endif /* UNIV_DEBUG || UNIV_HOTBACKUP */
@@ -314,7 +309,7 @@ enum recv_addr_state {
 
 /** Hashed page file address struct */
 struct recv_addr_t {
-  using List = UT_LIST_BASE_NODE_T(recv_t);
+  using List = UT_LIST_BASE_NODE_T(recv_t, rec_list);
 
   /** recovery state of the page */
   recv_addr_state state;
@@ -341,41 +336,37 @@ merge them and apply them to in-memory table objects finally */
 class MetadataRecover {
   using PersistentTables = std::map<
       table_id_t, PersistentTableMetadata *, std::less<table_id_t>,
-      ut_allocator<std::pair<const table_id_t, PersistentTableMetadata *>>>;
+      ut::allocator<std::pair<const table_id_t, PersistentTableMetadata *>>>;
 
  public:
   /** Default constructor */
-  MetadataRecover() UNIV_NOTHROW {}
+  MetadataRecover() UNIV_NOTHROW = default;
 
   /** Destructor */
   ~MetadataRecover();
 
   /** Parse a dynamic metadata redo log of a table and store
   the metadata locally
-  @param[in]	id	table id
-  @param[in]	version	table dynamic metadata version
-  @param[in]	ptr	redo log start
-  @param[in]	end	end of redo log
+  @param[in]    id      table id
+  @param[in]    version table dynamic metadata version
+  @param[in]    ptr     redo log start
+  @param[in]    end     end of redo log
   @retval ptr to next redo log record, nullptr if this log record
   was truncated */
   byte *parseMetadataLog(table_id_t id, uint64_t version, byte *ptr, byte *end);
 
-  /** Apply the collected persistent dynamic metadata to in-memory
-  table objects */
-  void apply();
-
   /** Store the collected persistent dynamic metadata to
   mysql.innodb_dynamic_metadata */
   void store();
 
   /** If there is any metadata to be applied
-  @return	true if any metadata to be applied, otherwise false */
-  bool empty() const { return (m_tables.empty()); }
+  @return       true if any metadata to be applied, otherwise false */
+  bool empty() const { return m_tables.empty(); }
 
  private:
   /** Get the dynamic metadata of a specified table,
   create a new one if not exist
-  @param[in]	id	table id
+  @param[in]    id      table id
   @return the metadata of the specified table */
   PersistentTableMetadata *getMetadata(table_id_t id);
 
@@ -393,7 +384,7 @@ struct recv_sys_t {
   /** Every space has its own heap and pages that belong to it. */
   struct Space {
     /** Constructor
-    @param[in,out]	heap	Heap to use for the log records. */
+    @param[in,out]      heap    Heap to use for the log records. */
     explicit Space(mem_heap_t *heap) : m_heap(heap), m_pages() {}
 
     /** Default constructor */
@@ -428,6 +419,104 @@ struct recv_sys_t {
 
   using Encryption_Keys = std::vector<Encryption_Key>;
 
+  /** Mini transaction log record. */
+  struct Mlog_record {
+    /* Space ID */
+    space_id_t space_id;
+    /* Page number */
+    page_no_t page_no;
+    /* Log type */
+    mlog_id_t type;
+    /* Log body */
+    const byte *body;
+    /* Record size */
+    size_t size;
+  };
+
+  using Mlog_records = std::vector<Mlog_record, ut::allocator<Mlog_record>>;
+
+  /** While scanning logs for multi-record mini-transaction (mtr), we have two
+  passes. In first pass, we check if all the logs of the mtr is present in
+  current recovery buffer or not. If yes, then in second pass we go through the
+  logs again the add to hash table for apply. To avoid parsing multiple times,
+  we save the parsed records in first pass and reuse them in second pass.
+
+  Parsing of redo log takes significant amount of time and this optimization of
+  avoiding second parse gave about 1.8x speed up on recovery scan time of 1G of
+  redo log from sysbench rw test.
+
+  There is currently no limit for maximum number of logs in an mtr. Practically,
+  from sysbench rw test recovery with 1G of redo log to recover from the record
+  count were spread from 3 - 1235 with majority between 600 - 700. So, it is
+  likely by saving 1k records we could avoid most of the re-parsing overhead.
+  Considering possible bigger number of records in other load and future changes
+  the limit for number of saved records is kept at 8k. The same value from the
+  contribution patch. The memory requirement 32 x 8k = 256k seems fine as one
+  time overhead for the entire instance.  */
+  static constexpr size_t MAX_SAVED_MLOG_RECS = 8 * 1024;
+
+  /** Save mlog record information. Silently returns if cannot save. Works only
+  in single threaded recovery scanner.
+  @param[in]    rec_num         record number in multi record group
+  @param[in]    space_id        space ID for the log record
+  @param[in]    page_no         page number for the log record
+  @param[in]    type            log record type
+  @param[in]    body            pointer to log record body in recovery buffer
+  @param[in]    len             length of the log record */
+  void save_rec(size_t rec_num, space_id_t space_id, page_no_t page_no,
+                mlog_id_t type, const byte *body, size_t len) {
+    /* No more space to save log. */
+    if (rec_num >= MAX_SAVED_MLOG_RECS) {
+      return;
+    }
+
+    ut_ad(rec_num < saved_recs.size());
+
+    if (rec_num >= saved_recs.size()) {
+      return;
+    }
+
+    auto &saved_rec = saved_recs[rec_num];
+
+    saved_rec.space_id = space_id;
+    saved_rec.page_no = page_no;
+    saved_rec.type = type;
+    saved_rec.body = body;
+    saved_rec.size = len;
+  }
+
+  /** Return saved mlog record information, if there. Works only
+  in single threaded recovery scanner.
+  @param[in]    rec_num         record number in multi record group
+  @param[out]   space_id        space ID for the log record
+  @param[out]   page_no         page number for the log record
+  @param[out]   type            log record type
+  @param[out]   body            pointer to log record body in recovery buffer
+  @param[out]   len             length of the log record
+  @return true iff saved record data is found. */
+  bool get_saved_rec(size_t rec_num, space_id_t &space_id, page_no_t &page_no,
+                     mlog_id_t &type, byte *&body, size_t &len) {
+    if (rec_num >= MAX_SAVED_MLOG_RECS) {
+      return false;
+    }
+
+    ut_ad(rec_num < saved_recs.size());
+
+    if (rec_num >= saved_recs.size()) {
+      return false;
+    }
+
+    auto &saved_rec = saved_recs[rec_num];
+
+    space_id = saved_rec.space_id;
+    page_no = saved_rec.page_no;
+    type = saved_rec.type;
+    body = const_cast<byte *>(saved_rec.body);
+    len = saved_rec.size;
+
+    return true;
+  }
+
 #ifndef UNIV_HOTBACKUP
 
   /*!< mutex protecting the fields apply_log_recs, n_addrs, and the
@@ -460,12 +549,6 @@ struct recv_sys_t {
   /** This is true when a log rec application batch is running */
   bool apply_batch_on;
 
-  /** Possible incomplete last recovered log block */
-  byte *last_block;
-
-  /** The nonaligned start address of the preceding buffer */
-  byte *last_block_buf_start;
-
   /** Buffer for parsing log records */
   byte *buf;
 
@@ -489,9 +572,8 @@ struct recv_sys_t {
   /** The log data has been scanned up to this lsn */
   lsn_t scanned_lsn;
 
-  /** The log data has been scanned up to this checkpoint
-  number (lowest 4 bytes) */
-  ulint scanned_checkpoint_no;
+  /** The log data has been scanned up to this epoch_no */
+  uint32_t scanned_epoch_no;
 
   /** Start offset of non-parsed log records in buf */
   ulint recovered_offset;
@@ -508,7 +590,7 @@ struct recv_sys_t {
   /** Tracks what should be the proper value of first_rec_group field in the
   header of the block to which recovered_lsn belongs. It might be also zero,
   in which case it means we do not know. */
-  uint32_t last_block_first_rec_group;
+  lsn_t last_block_first_mtr_boundary{};
 
   /** Set when finding a corrupt log block or record, or there
   is a log parsing buffer overflow */
@@ -518,11 +600,11 @@ struct recv_sys_t {
   is detected during log scan or apply */
   bool found_corrupt_fs;
 
-  /** If the recovery is from a cloned database. */
+  /** Data directory has been recognized as cloned data directory. */
   bool is_cloned_db;
 
-  /** Recovering from MEB. */
-  bool is_meb_recovery;
+  /** Data directory has been recognized as data directory from MEB. */
+  bool is_meb_db;
 
   /** Doublewrite buffer state before MEB recovery starts. We restore to this
   state after MEB recovery completes and disable the doublewrite buffer during
@@ -550,48 +632,47 @@ struct recv_sys_t {
 
   /** Tablespace IDs that were explicitly deleted. */
   Missing_Ids deleted;
+
+  /* Saved log records to avoid second round parsing log. */
+  Mlog_records saved_recs;
 };
 
 /** The recovery system */
 extern recv_sys_t *recv_sys;
 
-/** TRUE when applying redo log records during crash recovery; FALSE
-otherwise.  Note that this is FALSE while a background thread is
+/** true when applying redo log records during crash recovery; false
+otherwise.  Note that this is false while a background thread is
 rolling back incomplete transactions. */
 extern volatile bool recv_recovery_on;
 
-/** If the following is TRUE, the buffer pool file pages must be invalidated
-after recovery and no ibuf operations are allowed; this becomes TRUE if
+/** If the following is true, the buffer pool file pages must be invalidated
+after recovery and no ibuf operations are allowed; this becomes true if
 the log record hash table becomes too full, and log records must be merged
 to file pages already before the recovery is finished: in this case no
 ibuf operations are allowed, as they could modify the pages read in the
 buffer pool before the pages have been recovered to the up-to-date state.
 
-TRUE means that recovery is running and no operations on the log files
+true means that recovery is running and no operations on the log files
 are allowed yet: the variable name is misleading. */
 extern bool recv_no_ibuf_operations;
 
-/** TRUE when recv_init_crash_recovery() has been called. */
+/** true when recv_init_crash_recovery() has been called. */
 extern bool recv_needed_recovery;
 
-/** TRUE if buf_page_is_corrupted() should check if the log sequence
-number (FIL_PAGE_LSN) is in the future.  Initially FALSE, and set by
+/** true if buf_page_is_corrupted() should check if the log sequence
+number (FIL_PAGE_LSN) is in the future.  Initially false, and set by
 recv_recovery_from_checkpoint_start(). */
 extern bool recv_lsn_checks_on;
 
 /** Size of the parsing buffer; it must accommodate RECV_SCAN_SIZE many
 times! */
-#define RECV_PARSING_BUF_SIZE (2 * 1024 * 1024)
+constexpr uint32_t RECV_PARSING_BUF_SIZE = 2 * 1024 * 1024;
 
 /** Size of block reads when the log groups are scanned forward to do a
 roll-forward */
 #define RECV_SCAN_SIZE (4 * UNIV_PAGE_SIZE)
 
-/** This many frames must be left free in the buffer pool when we scan
-the log and store the scanned log records in the buffer pool: we will
-use these free frames to read in pages when we start applying the
-log records to the database. */
-extern ulint recv_n_pool_free_frames;
+extern size_t recv_n_frames_for_pages_per_pool_instance;
 
 /** A list of tablespaces for which (un)encryption process was not
 completed before crash. */
