@@ -31,7 +31,7 @@
 #include <new>
 
 #include "lex_string.h"
-#include "m_ctype.h"
+#include "m_string.h"
 #include "my_bit.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
@@ -40,6 +40,9 @@
 #include "myisam.h"
 #include "myisampack.h"
 #include "mysql/plugin.h"
+#include "mysql/strings/int2str.h"
+#include "mysql/strings/m_ctype.h"
+#include "nulls.h"
 #include "sql/current_thd.h"
 #include "sql/derror.h"
 #include "sql/field.h"
@@ -54,6 +57,7 @@
 #include "sql/system_variables.h"
 #include "storage/myisam/myisamdef.h"
 #include "storage/myisam/rt_index.h"
+#include "strxmov.h"
 
 #include "mysql/components/services/log_builtins.h"
 
