@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -173,7 +173,9 @@ void Dbtup::execDBINFO_SCANREQ(Signal* signal)
 
     const size_t num_config_params =
       sizeof(pools[0].config_params) / sizeof(pools[0].config_params[0]);
+    const Uint32 numPools = NDB_ARRAY_SIZE(pools);
     Uint32 pool = cursor->data[0];
+    ndbrequire(pool < numPools);
     BlockNumber bn = blockToMain(number());
     while(pools[pool].poolname)
     {
@@ -323,7 +325,8 @@ Dbtup::execDUMP_STATE_ORD(Signal* signal)
     Uint32 seed = (Uint32)time(0);
     if (signal->getLength() > 1)
       seed = signal->theData[1];
-    ndbout_c("Startar modul test av Page Manager (seed: 0x%x)", seed);
+    g_eventLogger->info("Startar modul test av Page Manager (seed: 0x%x)",
+                        seed);
     srand(seed);
 
     Vector<Chunk> chunks;
@@ -352,7 +355,8 @@ Dbtup::execDUMP_STATE_ORD(Signal* signal)
       }
       
       if (type == 1211)
-        ndbout_c("loop=%d case=%d free=%d alloc=%d", i, c, free, alloc);
+        g_eventLogger->info("loop=%d case=%d free=%d alloc=%d", i, c, free,
+                            alloc);
 
       if (type == 1213)
       {
@@ -369,7 +373,6 @@ Dbtup::execDUMP_STATE_ORD(Signal* signal)
 	break;
       case 2: { // Seize(n) - fail
 	alloc += free;
-	// Fall through
         sum_req += free;
         goto doalloc;
       }
@@ -383,14 +386,15 @@ Dbtup::execDUMP_STATE_ORD(Signal* signal)
 	  chunks.push_back(chunk);
 	  if(chunk.pageCount != alloc) {
 	    if (type == 1211)
-              ndbout_c("  Tried to allocate %d - only allocated %d - free: %d",
-                       alloc, chunk.pageCount, free);
-	  }
-	} else {
-	  ndbout_c("  Failed to alloc %d pages with %d pages free",
-		   alloc, free);
-	}
-	
+              g_eventLogger->info(
+                  "  Tried to allocate %d - only allocated %d - free: %d",
+                  alloc, chunk.pageCount, free);
+          }
+        } else {
+          g_eventLogger->info("  Failed to alloc %d pages with %d pages free",
+                              alloc, free);
+        }
+
         sum_conf += chunk.pageCount;
         Uint32 tot = fc_left + fc_right + fc_remove;
         sum_loop += tot;
@@ -417,9 +421,9 @@ Dbtup::execDUMP_STATE_ORD(Signal* signal)
       chunks.erase(chunks.size() - 1);
     }
 
-    ndbout_c("Got %u%% of requested allocs, loops : %u 100*avg: %u max: %u",
-             (100 * sum_conf) / sum_req, sum_loop, 100*sum_loop / LOOPS,
-             max_loop);
+    g_eventLogger->info(
+        "Got %u%% of requested allocs, loops : %u 100*avg: %u max: %u",
+        (100 * sum_conf) / sum_req, sum_loop, 100 * sum_loop / LOOPS, max_loop);
   }
 #endif
 
@@ -538,7 +542,7 @@ void Dbtup::printoutTuplePage(Uint32 fragid, Uint32 pageid, Uint32 printLimit)
   FragrecordPtr tmpFragP;
   TablerecPtr tmpTableP;
 
-  c_page_pool.getPtr(tmpPageP, pageid);
+  ndbrequire(c_page_pool.getPtr(tmpPageP, pageid));
 
   tmpFragP.i = fragid;
   ptrCheckGuard(tmpFragP, cnoOfFragrec, fragrecord);
@@ -600,12 +604,20 @@ template class Vector<Chunk>;
 NdbOut&
 operator<<(NdbOut& out, const Local_key & key)
 {
-  out << "[ m_page_no: " << dec << key.m_page_no 
-      << " m_file_no: " << dec << key.m_file_no 
-      << " m_page_idx: " << dec << key.m_page_idx << "]";
+  out << "[ m_page_no: " << dec << key.m_page_no << " m_file_no: " << dec
+      << key.m_file_no << " m_page_idx: " << dec << key.m_page_idx << "]";
   return out;
 }
 
+char*
+printLocal_Key(char buf[], int bufsize, const Local_key& key)
+{
+  BaseString::snprintf(buf, bufsize,
+                       "[ m_page_no: %u m_file_no: %u m_page_idx: %u ]",
+                       key.m_page_no, key.m_file_no, key.m_page_idx);
+  return buf;
+}
+
 static
 NdbOut&
 operator<<(NdbOut& out, const Dbtup::Tablerec::Tuple_offsets& off)
