@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,7 +36,6 @@
 
 // big brother
 #include <dbtup/Dbtup.hpp>
-#include <dblqh/Dblqh.hpp>
 #include <dbacc/Dbacc.hpp>
 
 // packed index keys and bounds
@@ -71,6 +70,7 @@
 
 class Configuration;
 struct mt_BuildIndxCtx;
+class Dblqh;
 
 class Dbtux : public SimulatedBlock {
   friend class Dbqtux;
@@ -95,25 +95,25 @@ class Dbtux : public SimulatedBlock {
   Dblqh* c_lqh;
   Dbacc* c_acc;
   void execTUX_BOUND_INFO(Signal* signal);
-  void execREAD_PSEUDO_REQ(Signal* signal);
+  void execREAD_PSEUDO_REQ(Uint32 scanPtrI, Uint32 attrId, Uint32* out, Uint32 out_words);
 
 private:
   // sizes are in words (Uint32)
-  STATIC_CONST( MaxIndexFragments = MAX_FRAG_PER_LQH );
-  STATIC_CONST( MaxIndexAttributes = MAX_ATTRIBUTES_IN_INDEX );
-  STATIC_CONST( MaxAttrDataSize = 2 * MAX_ATTRIBUTES_IN_INDEX + MAX_KEY_SIZE_IN_WORDS );
-  STATIC_CONST( MaxXfrmDataSize = MaxAttrDataSize * MAX_XFRM_MULTIPLY);
+  static constexpr Uint32 MaxIndexFragments = MAX_FRAG_PER_LQH;
+  static constexpr Uint32 MaxIndexAttributes = MAX_ATTRIBUTES_IN_INDEX;
+  static constexpr Uint32 MaxAttrDataSize = 2 * MAX_ATTRIBUTES_IN_INDEX + MAX_KEY_SIZE_IN_WORDS;
+  static constexpr Uint32 MaxXfrmDataSize = MaxAttrDataSize * MAX_XFRM_MULTIPLY;
 public:
-  STATIC_CONST( DescPageSize = 512 );
+  static constexpr Uint32 DescPageSize = 512;
 private:
-  STATIC_CONST( MaxTreeNodeSize = MAX_TTREE_NODE_SIZE );
-  STATIC_CONST( MaxPrefSize = MAX_TTREE_PREF_SIZE );
-  STATIC_CONST( ScanBoundSegmentSize = 7 );
-  STATIC_CONST( MaxAccLockOps = MAX_PARALLEL_OP_PER_SCAN );
-  STATIC_CONST( MaxTreeDepth = 32 );    // strict
+  static constexpr Uint32 MaxTreeNodeSize = MAX_TTREE_NODE_SIZE;
+  static constexpr Uint32 MaxPrefSize = MAX_TTREE_PREF_SIZE;
+  static constexpr Uint32 ScanBoundSegmentSize = 7;
+  static constexpr Uint32 MaxAccLockOps = MAX_PARALLEL_OP_PER_SCAN;
+  static constexpr Uint32 MaxTreeDepth = 32;    // strict
 #ifdef VM_TRACE
   // for TuxCtx::c_debugBuffer
-  STATIC_CONST( DebugBufferBytes = (MaxAttrDataSize << 2) );
+  static constexpr Uint32 DebugBufferBytes = (MaxAttrDataSize << 2);
 #endif
   BLOCK_DEFINES(Dbtux);
 
@@ -121,13 +121,13 @@ private:
   struct TuxCtx;
 
   // AttributeHeader size is assumed to be 1 word
-  STATIC_CONST( AttributeHeaderSize = 1 );
+  static constexpr Uint32 AttributeHeaderSize = 1;
 
   /*
    * Logical tuple address, "local key".  Identifies table tuples.
    */
   typedef Uint32 TupAddr;
-  STATIC_CONST( NullTupAddr = (Uint32)-1 );
+  static constexpr Uint32 NullTupAddr = (Uint32)-1;
 
   /*
    * Physical tuple address in TUP.  Provides fast access to table tuple
@@ -178,7 +178,7 @@ private:
     bool eq(const TreeEnt ent) const;
     int cmp(const TreeEnt ent) const;
   };
-  STATIC_CONST( TreeEntSize = sizeof(TreeEnt) >> 2 );
+  static constexpr Uint32 TreeEntSize = sizeof(TreeEnt) >> 2;
   static const TreeEnt NullTreeEnt;
 
   /*
@@ -204,7 +204,7 @@ private:
     Uint32 m_nodeScanInstance;
     TreeNode();
   };
-  STATIC_CONST( NodeHeadSize = sizeof(TreeNode) >> 2 );
+  static constexpr Uint32 NodeHeadSize = sizeof(TreeNode) >> 2;
 
   /*
    * Tree header.  There is one in each fragment.  Contains tree
@@ -274,11 +274,11 @@ private:
     Uint16 m_magic;
     enum { Magic = 0xDE5C };
   };
-  STATIC_CONST( DescHeadSize = sizeof(DescHead) >> 2 );
+  static constexpr Uint32 DescHeadSize = sizeof(DescHead) >> 2;
 
   typedef NdbPack::Type KeyType;
   typedef NdbPack::Spec KeySpec;
-  STATIC_CONST( KeyTypeSize = sizeof(KeyType) >> 2 );
+  static constexpr Uint32 KeyTypeSize = sizeof(KeyType) >> 2;
 
   typedef NdbPack::DataC KeyDataC;
   typedef NdbPack::Data KeyData;
@@ -296,7 +296,7 @@ private:
   typedef DataBufferSegment<ScanBoundSegmentSize, RT_DBTUX_SCAN_BOUND>
             ScanBoundSegment;
   typedef TransientPool<ScanBoundSegment> ScanBoundBuffer_pool;
-  STATIC_CONST(DBTUX_SCAN_BOUND_TRANSIENT_POOL_INDEX = 2);
+  static constexpr Uint32 DBTUX_SCAN_BOUND_TRANSIENT_POOL_INDEX = 2;
   typedef DataBuffer<ScanBoundSegmentSize,
                      ScanBoundBuffer_pool,
                      RT_DBTUX_SCAN_BOUND> ScanBoundBuffer;
@@ -313,7 +313,7 @@ private:
 
   // ScanLock
   struct ScanLock {
-    STATIC_CONST( TYPE_ID = RT_DBTUX_SCAN_LOCK);
+    static constexpr Uint32 TYPE_ID = RT_DBTUX_SCAN_LOCK;
     Uint32 m_magic;
 
     ScanLock() :
@@ -330,7 +330,7 @@ private:
     };
     Uint32 prevList;
   };
-  STATIC_CONST(DBTUX_SCAN_LOCK_TRANSIENT_POOL_INDEX = 1);
+  static constexpr Uint32 DBTUX_SCAN_LOCK_TRANSIENT_POOL_INDEX = 1;
   typedef Ptr<ScanLock> ScanLockPtr;
   typedef TransientPool<ScanLock> ScanLock_pool;
   typedef DLFifoList<ScanLock_pool> ScanLock_fifo;
@@ -361,7 +361,7 @@ private:
    * protocol is still followed until scan close.
    */
   struct ScanOp {
-    STATIC_CONST( TYPE_ID = RT_DBTUX_SCAN_OPERATION);
+    static constexpr Uint32 TYPE_ID = RT_DBTUX_SCAN_OPERATION;
     Uint32 m_magic;
 
     ~ScanOp()
@@ -375,7 +375,7 @@ private:
       Found = 3,                // return current as next scan result
       Blocked = 4,              // found and waiting for ACC lock
       Locked = 5,               // found and locked or no lock needed
-      Next = 6,                 // looking for next extry
+      Next = 6,                 // looking for next entry
       Last = 7,                 // after last entry
       Aborting = 8
     };
@@ -413,7 +413,7 @@ private:
     Uint32 prevList;
     ScanOp();
   };
-  STATIC_CONST(DBTUX_SCAN_OPERATION_TRANSIENT_POOL_INDEX = 0);
+  static constexpr Uint32 DBTUX_SCAN_OPERATION_TRANSIENT_POOL_INDEX = 0;
   typedef Ptr<ScanOp> ScanOpPtr;
   typedef TransientPool<ScanOp> ScanOp_pool;
   typedef DLList<ScanOp_pool> ScanOp_list;
@@ -652,7 +652,7 @@ private:
                     TreeEnt ent,
                     Uint32 count,
                     Uint32 *outputBuffer);
-  void readTablePk(TreeEnt ent, Uint32* pkData, unsigned& pkSize);
+  void readTableHashKey(TreeEnt ent, Uint32* pkData, unsigned& pkSize);
   void unpackBound(Uint32* const outputBuffer,
                    const ScanBound& bound,
                    KeyBoundC& searchBound);
@@ -736,7 +736,7 @@ private:
   void linkScan(NodeHandle& node, ScanOpPtr scanPtr, Uint32 scanInstance);
   void unlinkScan(NodeHandle& node, ScanOpPtr scanPtr, Uint32 scanInstance);
   bool islinkScan(NodeHandle& node, ScanOpPtr scanPtr, Uint32 scanInstance);
-  void relinkScan(ScanOp&, Frag&, bool need_lock = true, Uint32 line = 0);
+  void relinkScan(ScanOp&, Uint32 scanInstance, Frag&, bool need_lock = true, Uint32 line = 0);
 
   /*
    * DbtuxTree.cpp
@@ -830,9 +830,6 @@ private:
                            Dbtux *tux_block);
   void prepare_move_scan_ctx(ScanOpPtr scanPtr, Dbtux *tux_block);
 
-  /*
-   * DbtuxCmp.cpp
-   */
   int cmpSearchKey(TuxCtx&, const KeyDataC& searchKey, const KeyDataC& entryKey, Uint32 cnt);
   int cmpSearchBound(TuxCtx&, const KeyBoundC& searchBound, const KeyDataC& entryKey, Uint32 cnt);
 
@@ -840,14 +837,14 @@ private:
    * DbtuxStat.cpp
    */
   // one-round-trip tree-dive records in range
-  void statRecordsInRange(ScanOpPtr scanPtr, Uint32* out);
+  void statRecordsInRange(ScanOpPtr scanPtr, Uint32* out, Uint32 out_words);
   Uint32 getEntriesBeforeOrAfter(Frag& frag, TreePos pos, unsigned idir);
   unsigned getPathToNode(NodeHandle node, Uint16* path);
   // stats scan
   int statScanInit(StatOpPtr, const Uint32* data, Uint32 len, Uint32* usedLen);
   int statScanAddRow(StatOpPtr, TreeEnt ent);
-  void statScanReadKey(StatOpPtr, Uint32* out);
-  void statScanReadValue(StatOpPtr, Uint32* out);
+  void statScanReadKey(StatOpPtr, Uint32* out, Uint32 out_words);
+  void statScanReadValue(StatOpPtr, Uint32* out, Uint32 out_words);
   void execINDEX_STAT_REP(Signal*); // from TRIX
   // stats monitor request
   void execINDEX_STAT_IMPL_REQ(Signal*);
@@ -901,8 +898,8 @@ private:
     DebugLock = 16,             // log ACC locks
     DebugStat = 32              // log stats collection
   };
-  STATIC_CONST( DataFillByte = 0xa2 );
-  STATIC_CONST( NodeFillByte = 0xa4 );
+  static constexpr Uint32 DataFillByte = 0xa2;
+  static constexpr Uint32 NodeFillByte = 0xa4;
 #endif
 
   void execDBINFO_SCANREQ(Signal* signal);
@@ -962,6 +959,38 @@ private:
 #ifdef VM_TRACE
     char* c_debugBuffer;
 #endif
+    // function for clearing context
+    void reset()
+    {
+      // jamBuffer left
+      scanPtr.i = RNIL;
+      scanPtr.p = nullptr;
+      fragPtr.i = RNIL;
+      fragPtr.p = nullptr;
+      indexPtr.i = RNIL;
+      indexPtr.p = nullptr;
+      tupIndexFragPtr = nullptr;
+      tupIndexTablePtr = nullptr;
+      tupRealFragPtr = nullptr;
+      tupRealTablePtr = nullptr;
+      attrDataOffset = 0;
+      tuxFixHeaderSize = 0;
+      // searchScanDataArray left
+      // searchScanBoundArray left
+      keyAttrs = nullptr;
+      // searchKeyDataArray left
+      // searchKeyBoundArray left
+      scanBoundCnt = 0;
+      descending = 0;
+      m_current_ent.m_tupLoc = NullTupLoc;
+      m_current_ent.m_tupVersion = 0;
+      // c_searchKey left
+      // c_nextKey left
+      // c_entryKey left
+      // c_dataBuffer left
+      // c_boundBuffer left
+      // c_debugBuffer left
+    }
   };
 
   struct TuxCtx c_ctx; // Global Tux context, for everything build MT-index build
@@ -1012,6 +1041,7 @@ private:
   Uint32 get_my_scan_instance();
   Uint32 get_block_from_scan_instance(Uint32);
   Uint32 get_instance_from_scan_instance(Uint32);
+  bool checkScanInstance(Uint32 scanInstance);
 public:
   static Uint64 getTransactionMemoryNeed(
     const Uint32 ldm_instance_count,
@@ -1680,8 +1710,6 @@ Dbtux::max(unsigned x, unsigned y)
   return x > y ? x : y;
 }
 
-// DbtuxCmp.cpp
-
 /**
  * Can be called from MT-build of ordered indexes,
  * but it doesn't make use of the MT-context other
@@ -1742,12 +1770,10 @@ inline
 void
 Dbtux::relinkScan(Uint32 line)
 {
-  if (c_ctx.scanPtr.p != nullptr)
-  {
-    ScanOp& scan = *c_ctx.scanPtr.p;
-    Frag& frag = *c_ctx.fragPtr.p;
-    relinkScan(scan, frag, true, line);
-  }
+  ndbrequire(c_ctx.scanPtr.p != nullptr);
+  ScanOp& scan = *c_ctx.scanPtr.p;
+  Frag& frag = *c_ctx.fragPtr.p;
+  relinkScan(scan, m_my_scan_instance, frag, true, line);
 }
 #undef JAM_FILE_ID
 
