@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,35 +33,32 @@
 // there is no filtering feature (yet) like "DebugStat"
 
 void
-Dbtux::execREAD_PSEUDO_REQ(Signal* signal)
+Dbtux::execREAD_PSEUDO_REQ(Uint32 scanPtrI, Uint32 attrId, Uint32* out, Uint32 out_words)
 {
   jamEntry();
   ScanOpPtr scanPtr;
-  scanPtr.i = signal->theData[0];
+  scanPtr.i = scanPtrI;
   ndbrequire(c_scanOpPool.getValidPtr(scanPtr));
   StatOpPtr statPtr;
   statPtr.i = scanPtr.p->m_statOpPtrI;
 
-  Uint32 attrId = signal->theData[1];
-  Uint32* out = &signal->theData[0];
-
   switch (attrId) {
   case AttributeHeader::RECORDS_IN_RANGE:
     jam();
     ndbrequire(statPtr.i == RNIL);
-    statRecordsInRange(scanPtr, out);
+    statRecordsInRange(scanPtr, out, out_words);
     break;
   case AttributeHeader::INDEX_STAT_KEY:
     jam();
     ndbrequire(statPtr.i != RNIL);
     c_statOpPool.getPtr(statPtr);
-    statScanReadKey(statPtr, out);
+    statScanReadKey(statPtr, out, out_words);
     break;
   case AttributeHeader::INDEX_STAT_VALUE:
     jam();
     ndbrequire(statPtr.i != RNIL);
     c_statOpPool.getPtr(statPtr);
-    statScanReadValue(statPtr, out);
+    statScanReadValue(statPtr, out, out_words);
     break;
   default:
     ndbabort();
@@ -81,7 +78,7 @@ Dbtux::execREAD_PSEUDO_REQ(Signal* signal)
  * 3) after range.  1-3) are estimates and need not add up to 0).
  */
 void
-Dbtux::statRecordsInRange(ScanOpPtr scanPtr, Uint32* out)
+Dbtux::statRecordsInRange(ScanOpPtr scanPtr, Uint32* out, Uint32 out_words)
 {
   ScanOp& scan = *scanPtr.p;
   Frag& frag = *c_fragPool.getPtr(scan.m_fragPtrI);
@@ -110,6 +107,7 @@ Dbtux::statRecordsInRange(ScanOpPtr scanPtr, Uint32* out)
     ndbrequire(pos2.m_loc != NullTupLoc);
   }
   // wl4124_todo change all to Uint64 if ever needed (unlikely)
+  ndbassert(4 <= out_words);
   out[0] = (Uint32)frag.m_entryCount;
   out[2] = getEntriesBeforeOrAfter(frag, pos1, 0);
   out[3] = getEntriesBeforeOrAfter(frag, pos2, 1);
@@ -425,7 +423,7 @@ Dbtux::statScanAddRow(StatOpPtr statPtr, TreeEnt ent)
 }
 
 void
-Dbtux::statScanReadKey(StatOpPtr statPtr, Uint32* out)
+Dbtux::statScanReadKey(StatOpPtr statPtr, Uint32* out, Uint32 out_words)
 {
   StatOp& stat = *statPtr.p;
   int ret;
@@ -435,11 +433,12 @@ Dbtux::statScanReadKey(StatOpPtr statPtr, Uint32* out)
   ndbrequire(ret == 0);
   D("statScanReadKey" << V(keyData));
   keyData.convert(NdbPack::Endian::Little);
+  ndbrequire(keyData.get_full_len() <= out_words * 4);
   memcpy(out, keyData.get_full_buf(), keyData.get_full_len());
 }
 
 void
-Dbtux::statScanReadValue(StatOpPtr statPtr, Uint32* out)
+Dbtux::statScanReadValue(StatOpPtr statPtr, Uint32* out, Uint32 out_words)
 {
   StatOp& stat = *statPtr.p;
   int ret;
@@ -468,6 +467,7 @@ Dbtux::statScanReadValue(StatOpPtr statPtr, Uint32* out)
 
   D("statScanReadValue" << V(valueData));
   valueData.convert(NdbPack::Endian::Little);
+  ndbrequire(valueData.get_full_len() <= out_words * 4);
   memcpy(out, valueData.get_full_buf(), valueData.get_full_len());
 }
 
@@ -652,6 +652,7 @@ Dbtux::statMonCheck(Signal* signal, StatMon& mon)
     {
       const Uint64 count = frag.m_entryCount;
       const Uint64 ops = frag.m_entryOps;
+      D("statMonCheck" << V(ops) << V(count));
       if (count <= 1)
       {
         jam();
@@ -661,16 +662,29 @@ Dbtux::statMonCheck(Signal* signal, StatMon& mon)
       else
       {
         jam();
-        // compute scaled percentags - see wl4124.txt
-        double a = c_indexStatTriggerPct;
-        double b = c_indexStatTriggerScale;
-        double c = (double)count;
-        double d = 1 + 0.01 * b * tux_log2(c); // inverse scale factor
-        double e = a / d; // scaled trigger pct
-        double f = (double)ops;
-        double g = 100.0 * f / c;
-        update = (g >= e);
-        D("statMonCheck" << V(update) << V(f) << V(c));
+        /*
+         * Check if the percent change in the index is high enough to trigger
+         * an update. The change is computed using the number of index
+         * operations since the last update compared to the number of entries.
+         * Note that this trigger percent threshold is scaled down
+         * logarithmically for large indexes.
+         */
+        const double triggerPercent = c_indexStatTriggerPct;
+
+        /*
+         * Compute scale factor depending on the number of entries and
+         * the IndexStatTriggerScale option and use it to scale down
+         * the trigger percent.
+         */
+        const double triggerScale = c_indexStatTriggerScale;
+        const double fragEntryCount = (double)count;
+        const double scaleFactor = 1 + 0.01 * triggerScale * tux_log2(fragEntryCount);
+        const double scaledTriggerPercent = triggerPercent / scaleFactor;
+
+        const double fragEntryOps = (double)ops;
+        const double indexPercentChange = 100.0 * fragEntryOps / fragEntryCount;
+        update = (indexPercentChange >= scaledTriggerPercent);
+        D("statMonCheck" << V(update) << V(indexPercentChange) << V(scaledTriggerPercent));
       }
     }
 
