@@ -35,7 +35,9 @@
 #include "m_string.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
+#include "mysql/strings/int2str.h"
 #include "sql/my_decimal.h"
+#include "template_utils.h"
 #include "unittest/gunit/benchmark.h"
 
 namespace decimal_unittest {
